/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved. 

Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
  
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. 
  
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
*/
				 
package orthotweezers;
import java.io.*;
import dk.eos.io.port.IOPort;

/**
 * Interface to the Advantech PCL-726 I/O board with 6 channel 12-bit D/A,
 * 16 digital inputs and 16 digital outputs.
 *
 * @author  unknown
 */
public class AdvantechPCL726 {

    /** Creates new AdvantechPCL726.  This sets all D/A outputs to initialDASet.
     *
     * @param base  base address to which to I/O card is set
     * @param initialDASet  the value to set all D/A channels with.
     */
    public AdvantechPCL726 (int base, int initialDASet) {
        // Create the 16 IOPorts
        for (int i = 0; i < 16; ++i)
            _port[i] = new IOPort (base + i);
        
        for (int i = 1; i <= 6; ++i) {
            daSet (i, initialDASet);
        }
    }
    
   /** Set the D/A channel.
     *
     * @param channel   the D/A channel to set.  Must be from 1 to 6 inclusive.
     *                  If channel is outside this range, this does nothing.
     * @param value     the 12-bit value to output, in the range 0 to 0x0fff.
     *                  Values outside this range will be clipped to this range.
     * @see #lastDASet
     */
    public void daSet (int channel, int value) {
        if (channel < 1 || channel > 6)
            // illegal channel number
            return;
        
        if (value < 0)
            value = 0;
        if (value > 0x0fff)
            value = 0x0fff;
        try {
            // Note that channel is indexed from 1 but _port[] is indexed from 0.
            // Do hi byte first.
            _port[(channel - 1) * 2].out (value >> 8);
            _port[(channel - 1) * 2 + 1].out (value & 0xff);
            
            _lastDASet[channel - 1] = value;
        } catch (IOException e) {
            // ignore error
        }
    }
    
    /** Get the last value sent to the D/A by daSet().
     *
     * @param channel   the D/A channel to query.  Must be from 1 to 6 inclusive.
     *                  If channel is outside this range, this returns -1.
     * @return      the last value sent to the given D/A channel by daSet(), or
     *              -1 if the channel number is out of range.
     * @see     #daSet
     */
    public int lastDASet (int channel) {
        if (channel < 1 || channel > 6)
            // illegal channel number
            return -1;
        
        return _lastDASet[channel - 1];
    }

    private int _lastDASet[] = new int[6];
    // The 16 ports starting at the base address.
    private IOPort _port[] = new IOPort[16];
}