/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved. 

Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
  
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. 
  
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
*/
				 
package orthotweezers;

/** A CancelThread extends Thread and allows a mechanism to cleanly cancel the thread.
 * When the thread is created its enable state is true (not to be confused with
 * the interrupted or isAlive state).  If the cancel() method is called, the enable
 * state is set to false.  A thread may call CancelThread.check() and if cancel() has
 * been called (by another thread) then check() throws a CancelException.
 *
 * @author  unknown
 */

public class CancelThread extends Thread {
    /** Creates a new CancelThread as in Thread(Runnable target)
     */
    public CancelThread (Runnable runnable) {
        super (runnable);
    }
    
    /** This sets the enabled state false so that the next call to check (presumably
     * by anther thread) will throw a CancelException.
     *
     * @see #check
     */
    public void cancel() {
        _enabled = false;
    }
    
    /** If this thread is still enabled, simply return.  But if
     * not enabled (because another thread called cancel) then this throws
     * a CancelException.  This is a static method and queries Thread.currentThread()
     * to see if the current thread is an instanceof CancelThread.  If not, it
     * simply returns, but if it is, then it looks at the enabled state as described.
     *
     * @see #cancel
     * @exception CancelException   if this object is no longer enabled
     */
    public static void check() throws CancelException {
        if (Thread.currentThread() instanceof CancelThread) {
            if (!((CancelThread)Thread.currentThread())._enabled) {
                throw new CancelException ("operation cancelled");
            }
        }
    }
    
    private boolean _enabled = true;
}
