/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved. 

Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
  
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. 
  
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
*/
				 
package orthotweezers;

import java.text.NumberFormat;

/** GotoXYZ is a modal dialog box which allows the user to enter stage XYZ
 * coordinates to go to and provides a Cancel button to interrupt it.
 *
 * @author  unknown
 */
public class GotoXYZ extends javax.swing.JDialog implements Runnable {
    /** Creates new modal form GotoXYZ with initial stage x, y and z values
     * as given.
     * The user can change the values and click Go To.  The user can cancel the motion.
     *
     * @param parent    The parent Frame
     * @param control   The OrthoTweezersControl used to control the system
     */
    public GotoXYZ(java.awt.Frame parent, OrthoTweezersControl control,
                   double x, double y, double z) {
        super (parent, true);
        _control = control;

        initComponents ();
        
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumFractionDigits (4);
        nf.setMaximumFractionDigits (4);

        _xField.setText (nf.format (x));
        _yField.setText (nf.format (y));
        _zField.setText (nf.format (z));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        _xField = new javax.swing.JTextField();
        _yField = new javax.swing.JTextField();
        _zField = new javax.swing.JTextField();
        _status = new javax.swing.JLabel();
        gotoButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        getContentPane().setLayout(null);
        setTitle("Enter X,Y,Z To Go To");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        }
        );
        
        jLabel1.setText("X (mm):");
        
        getContentPane().add(jLabel1);
        jLabel1.setBounds(10, 10, 50, 20);
        
        
        jLabel2.setText("Y (mm):");
        
        getContentPane().add(jLabel2);
        jLabel2.setBounds(10, 30, 50, 20);
        
        
        jLabel3.setText("Z (mm):");
        
        getContentPane().add(jLabel3);
        jLabel3.setBounds(10, 50, 50, 20);
        
        
        _xField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                gotoButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(_xField);
        _xField.setBounds(60, 10, 140, 20);
        
        
        _yField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                gotoButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(_yField);
        _yField.setBounds(60, 30, 140, 20);
        
        
        _zField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                gotoButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(_zField);
        _zField.setBounds(60, 50, 140, 20);
        
        
        _status.setText("Enter X,Y,Z and click Go To");
        
        getContentPane().add(_status);
        _status.setBounds(20, 80, 160, 20);
        
        
        gotoButton.setText("Go To");
        gotoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                gotoButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(gotoButton);
        gotoButton.setBounds(30, 110, 70, 20);
        
        
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(cancelButton);
        cancelButton.setBounds(110, 110, 80, 20);
        
        setBounds(0, 0, 250, 200);
    }//GEN-END:initComponents

  private void cancelButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
// Add your handling code here:
    doClose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void gotoButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_gotoButtonActionPerformed
// Add your handling code here:
    try {
        _x = Double.parseDouble (_xField.getText());
    } catch (NumberFormatException e) {
        _status.setText ("Type a valid number for X");
        // Just return and allow the user to try again.
        return;
    }
    try {
        _y = Double.parseDouble (_yField.getText());
    } catch (NumberFormatException e) {
        _status.setText ("Type a valid number for Y");
        // Just return and allow the user to try again.
        return;
    }
    try {
        _z = Double.parseDouble (_zField.getText());
    } catch (NumberFormatException e) {
        _status.setText ("Type a valid number for Z");
        // Just return and allow the user to try again.
        return;
    }

    // Should only be able to go once, so disable it.
    gotoButton.setEnabled (false);

    // Start the thread which will call run().
    _cancelThread = new CancelThread (this);
    _cancelThread.start();    
  }//GEN-LAST:event_gotoButtonActionPerformed

    /** This is run in a separate thread to go to _x _y _z.
     */
    public void run() {
        try {
            // As in OrthoTweezersFrame.moveTo, move Z first if it is to raise.
            boolean didZ = false;
            if (_z > _control.stageZ()) {
                // raising Z, so move Z first.
                _control.moveToZ (_z);
                didZ = true;
            }

            CancelThread.check();
            _control.moveToXY (_x, _y);
            CancelThread.check();

            if (!didZ)
            _control.moveToZ (_z);

            _status.setText ("done");
            cancelButton.setText ("Close");
        } catch (CancelException e) {
            // user cancelled.
            _status.setText (e.getMessage());
        }
    }

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose ();
    }//GEN-LAST:event_closeDialog

    private void doClose () {
        if (_cancelThread != null)
            // cancel the running thread
            _cancelThread.cancel();
        
        setVisible (false);
        dispose ();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JTextField _xField;
    private javax.swing.JTextField _yField;
    private javax.swing.JTextField _zField;
    private javax.swing.JLabel _status;
    private javax.swing.JButton gotoButton;
    private javax.swing.JButton cancelButton;
    // End of variables declaration//GEN-END:variables

    private double _x, _y, _z;
    private OrthoTweezersControl _control;
    private CancelThread _cancelThread;
}
