/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved. 

Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
  
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. 
  
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
*/
				 
package orthotweezers;

import java.io.*;
import java.util.Vector;
import java.net.URL;
import javax.vecmath.*;

/**
 * Main window for the OrthoTweezers user interface.
 *
 * @author  unknown
 */
public class OrthoTweezers extends javax.swing.JFrame {

    /** Creates new form OrthoTweezers.
     */
    public OrthoTweezers() {
        initComponents ();
        _control = new OrthoTweezersControl ();
        _tools = new OrthoTweezersTools (_control);

        // Create and show the modeless monitor dialog
        _monitor = new OrthoTweezersMonitor (this, _control);
        _monitor.show();

        // Find the filename of the preferences file.
        URL classURL = getClass().getResource("OrthoTweezers.class");
        if (classURL == null) {
            // This shouldn't really happen since the .class file should always exist
            // Debug: may need to handle case where the .class file is in a jar file
            _status.setText ("Cannot find OrthoTweezers.class file");
        }
        else {
            File classFile = new File (classURL.getPath());
            // Construct the preferences file path from the OrthoTweezers.class directory
            _preferencesFile = new File (classFile.getParent(), "orthotweezers.ini");

            // Only read preferences file if it exists.  If it doesn't,
            //   then it will be created by _preferences.saveToFile() upon exiting.
            if (_preferencesFile.exists()) {
                if (_preferences.loadFromFile (_preferencesFile) == false) {
                    _status.setText ("Error reading preferences file");
                }
                else {
                    // Initialize _control from Preferences.
                    _control.setFromPreferences (_preferences);

                    _edge1.x = _preferences.maybeGetDouble (Preferences.EDGE1_X, 0);
                    _edge1.y = _preferences.maybeGetDouble (Preferences.EDGE1_Y, 0);
                    _edge1.z = _preferences.maybeGetDouble (Preferences.EDGE1_Z, 0);
                    _pallet1.x = _preferences.maybeGetDouble (Preferences.PALLET1_X, 0);
                    _pallet1.y = _preferences.maybeGetDouble (Preferences.PALLET1_Y, 0);
                    _pallet1.z = _preferences.maybeGetDouble (Preferences.PALLET1_Z, 0);
                }
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        motionTimer = new org.netbeans.examples.lib.timerbean.Timer();
        jMenuBar1 = new javax.swing.JMenuBar();
        menuView = new javax.swing.JMenu();
        menuViewMonitor = new javax.swing.JMenuItem();
        menuScript = new javax.swing.JMenu();
        menuScriptAddFrame = new javax.swing.JMenuItem();
        menuScriptRunFrames = new javax.swing.JMenuItem();
        menuScripClearFrames = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        menuScriptSetEdge1 = new javax.swing.JMenuItem();
        menuScriptSetPallet1 = new javax.swing.JMenuItem();
        menuScriptGotoEdge1 = new javax.swing.JMenuItem();
        menuScriptGotoPallet1 = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        menuScriptGraspBlock = new javax.swing.JMenuItem();
        menuStage = new javax.swing.JMenu();
        menuStageGotoXYZ = new javax.swing.JMenuItem();
        menuStageResetXY = new javax.swing.JMenuItem();
        menuStageResetZ = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        menuStageIncreaseXToContact = new javax.swing.JMenuItem();
        menuStageDecreaseXToContact = new javax.swing.JMenuItem();
        menuStageIncreaseYToContact = new javax.swing.JMenuItem();
        menuStageDecreaseYToContact = new javax.swing.JMenuItem();
        menuTweezers = new javax.swing.JMenu();
        menuTweezersCalibrate = new javax.swing.JMenuItem();
        menuTweezersQuickCalibrate = new javax.swing.JMenuItem();
        menuTweezersGoto = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        menuTweezersIncreaseProbeXToContact = new javax.swing.JMenuItem();
        menuTweezersDecreaseProbeXToContact = new javax.swing.JMenuItem();
        menuTweezersIncreaseProbeYToContact = new javax.swing.JMenuItem();
        menuTweezersDecreaseProbeYToContact = new javax.swing.JMenuItem();
        xDecrease = new javax.swing.JButton();
        yIncrease = new javax.swing.JButton();
        zDecrease = new javax.swing.JButton();
        zIncrease = new javax.swing.JButton();
        xIncrease = new javax.swing.JButton();
        yDecrease = new javax.swing.JButton();
        _status = new javax.swing.JLabel();
        xIncYInc = new javax.swing.JButton();
        xDecYInc = new javax.swing.JButton();
        xIncYDec = new javax.swing.JButton();
        xDecYDec = new javax.swing.JButton();
        probeYInc = new javax.swing.JButton();
        probeYDec = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        probeXDec = new javax.swing.JButton();
        probeXInc = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        addFrame = new javax.swing.JButton();
        runFrames = new javax.swing.JButton();
        _frameCount = new javax.swing.JLabel();
        gotoXYZButton = new javax.swing.JButton();
        probeXDecYDec = new javax.swing.JButton();
        probeXIncYInc = new javax.swing.JButton();
        probeXIncYDec = new javax.swing.JButton();
        probeXDecYInc = new javax.swing.JButton();
        gotoTweezerButton = new javax.swing.JButton();
        motionTimer.setDelay(3L);
        motionTimer.addTimerListener(new org.netbeans.examples.lib.timerbean.TimerListener() {
            public void onTime(java.awt.event.ActionEvent evt) {
                motionTimerOnTime(evt);
            }
        }
        );
        
        menuView.setText("View");
          
          menuViewMonitor.setText("Monitor");
            menuViewMonitor.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuViewMonitorActionPerformed(evt);
                }
            }
            );
            menuView.add(menuViewMonitor);
            jMenuBar1.add(menuView);
          
        menuScript.setText("Script");
          
          menuScriptAddFrame.setText("Add Frame");
            menuScriptAddFrame.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptAddFrameActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptAddFrame);
            
          menuScriptRunFrames.setText("Run Frames...");
            menuScriptRunFrames.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptRunFramesActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptRunFrames);
            
          menuScripClearFrames.setText("Clear Frames");
            menuScripClearFrames.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScripClearFramesActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScripClearFrames);
            
          menuScript.add(jSeparator3);
            
          menuScriptSetEdge1.setText("Set Edge 1");
            menuScriptSetEdge1.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptSetEdge1ActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptSetEdge1);
            
          menuScriptSetPallet1.setText("Set Pallet 1 Origin");
            menuScriptSetPallet1.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptSetPallet1ActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptSetPallet1);
            
          menuScriptGotoEdge1.setText("Go To Edge 1...");
            menuScriptGotoEdge1.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptGotoEdge1ActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptGotoEdge1);
            
          menuScriptGotoPallet1.setText("Go To Pallet 1 Origin...");
            menuScriptGotoPallet1.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptGotoPallet1ActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptGotoPallet1);
            
          menuScript.add(jSeparator4);
            
          menuScriptGraspBlock.setText("Grasp Block");
            menuScriptGraspBlock.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptGraspBlockActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptGraspBlock);
            jMenuBar1.add(menuScript);
          
        menuStage.setText("Stage");
          
          menuStageGotoXYZ.setText("Go To XYZ...");
            menuStageGotoXYZ.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageGotoXYZActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageGotoXYZ);
            
          menuStageResetXY.setText("Reset XY");
            menuStageResetXY.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageResetXYActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageResetXY);
            
          menuStageResetZ.setText("Reset Z");
            menuStageResetZ.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageResetZActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageResetZ);
            
          menuStage.add(jSeparator2);
            
          menuStageIncreaseXToContact.setActionCommand("Increase X To Contact");
            menuStageIncreaseXToContact.setText("Increase X To Contact");
            menuStageIncreaseXToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageIncreaseXToContactActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageIncreaseXToContact);
            
          menuStageDecreaseXToContact.setActionCommand("Decrease X To Contact");
            menuStageDecreaseXToContact.setText("Decrease X To Contact");
            menuStageDecreaseXToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageDecreaseXToContactActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageDecreaseXToContact);
            
          menuStageIncreaseYToContact.setActionCommand("Increase Y To Contact");
            menuStageIncreaseYToContact.setText("Increase Y To Contact");
            menuStageIncreaseYToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageIncreaseYToContactActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageIncreaseYToContact);
            
          menuStageDecreaseYToContact.setActionCommand("Decrease Y To Contact");
            menuStageDecreaseYToContact.setText("Decrease Y To Contact");
            menuStageDecreaseYToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageDecreaseYToContactActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageDecreaseYToContact);
            jMenuBar1.add(menuStage);
          
        menuTweezers.setText("Tweezers");
          
          menuTweezersCalibrate.setText("Calibrate...");
            menuTweezersCalibrate.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersCalibrateActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersCalibrate);
            
          menuTweezersQuickCalibrate.setText("Quick Calibrate Tip Offsets");
            menuTweezersQuickCalibrate.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersQuickCalibrateActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersQuickCalibrate);
            
          menuTweezersGoto.setText("Goto Deflection...");
            menuTweezersGoto.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersGotoActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersGoto);
            
          menuTweezers.add(jSeparator1);
            
          menuTweezersIncreaseProbeXToContact.setText("Increase Probe X To Contact");
            menuTweezersIncreaseProbeXToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersIncreaseProbeXToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersIncreaseProbeXToContact);
            
          menuTweezersDecreaseProbeXToContact.setText("Decrease Probe X To Contact");
            menuTweezersDecreaseProbeXToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersDecreaseProbeXToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersDecreaseProbeXToContact);
            
          menuTweezersIncreaseProbeYToContact.setText("Increase Probe Y To Contact");
            menuTweezersIncreaseProbeYToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersIncreaseProbeYToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersIncreaseProbeYToContact);
            
          menuTweezersDecreaseProbeYToContact.setText("DecreaseProbeYToContact");
            menuTweezersDecreaseProbeYToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersDecreaseProbeYToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersDecreaseProbeYToContact);
            jMenuBar1.add(menuTweezers);
          getContentPane().setLayout(null);
        setTitle("Ortho Tweezers");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        }
        );
        
        xDecrease.setText("<");
        xDecrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xDecrease);
        xDecrease.setLocation(10, 70);
        xDecrease.setSize(xDecrease.getPreferredSize());
        
        
        yIncrease.setText("^");
        yIncrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                yIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(yIncrease);
        yIncrease.setLocation(50, 40);
        yIncrease.setSize(yIncrease.getPreferredSize());
        
        
        zDecrease.setText("Lower");
        zDecrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                zDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(zDecrease);
        zDecrease.setLocation(80, 10);
        zDecrease.setSize(zDecrease.getPreferredSize());
        
        
        zIncrease.setText("Raise");
        zIncrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                zIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(zIncrease);
        zIncrease.setLocation(10, 10);
        zIncrease.setSize(zIncrease.getPreferredSize());
        
        
        xIncrease.setText(">");
        xIncrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xIncrease);
        xIncrease.setLocation(100, 70);
        xIncrease.setSize(xIncrease.getPreferredSize());
        
        
        yDecrease.setText("v");
        yDecrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                yDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(yDecrease);
        yDecrease.setLocation(50, 100);
        yDecrease.setSize(yDecrease.getPreferredSize());
        
        
        _status.setText("initialized");
        
        getContentPane().add(_status);
        _status.setBounds(60, 200, 290, 20);
        
        
        xIncYInc.setText("/");
        xIncYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xIncreaseMousePressed(evt);
                yIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xIncYInc);
        xIncYInc.setLocation(100, 40);
        xIncYInc.setSize(xIncYInc.getPreferredSize());
        
        
        xDecYInc.setText("\\");
        xDecYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xDecreaseMousePressed(evt);
                yIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xDecYInc);
        xDecYInc.setLocation(10, 40);
        xDecYInc.setSize(xDecYInc.getPreferredSize());
        
        
        xIncYDec.setText("\\");
        xIncYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xIncreaseMousePressed(evt);
                yDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xIncYDec);
        xIncYDec.setLocation(100, 100);
        xIncYDec.setSize(xIncYDec.getPreferredSize());
        
        
        xDecYDec.setText("/");
        xDecYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xDecreaseMousePressed(evt);
                yDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xDecYDec);
        xDecYDec.setLocation(10, 100);
        xDecYDec.setSize(xDecYDec.getPreferredSize());
        
        
        probeYInc.setText("^");
        probeYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeYIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeYInc);
        probeYInc.setBounds(160, 20, 50, 30);
        
        
        probeYDec.setText("v");
        probeYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeYDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeYDec);
        probeYDec.setBounds(160, 70, 50, 30);
        
        
        jLabel2.setText("probe Y");
        
        getContentPane().add(jLabel2);
        jLabel2.setLocation(170, 50);
        jLabel2.setSize(jLabel2.getPreferredSize());
        
        
        probeXDec.setText("<");
        probeXDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXDec);
        probeXDec.setBounds(220, 110, 50, 30);
        
        
        probeXInc.setText(">");
        probeXInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXInc);
        probeXInc.setBounds(270, 110, 50, 30);
        
        
        jLabel3.setText("probe X");
        
        getContentPane().add(jLabel3);
        jLabel3.setLocation(250, 140);
        jLabel3.setSize(jLabel3.getPreferredSize());
        
        
        addFrame.setText("Add Frame");
        addFrame.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuScriptAddFrameActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(addFrame);
        addFrame.setBounds(10, 140, 110, 30);
        
        
        runFrames.setText("Run Frames...");
        runFrames.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuScriptRunFramesActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(runFrames);
        runFrames.setBounds(10, 170, 120, 30);
        
        
        _frameCount.setText("0 frames");
        
        getContentPane().add(_frameCount);
        _frameCount.setBounds(130, 140, 80, 20);
        
        
        gotoXYZButton.setToolTipText("Go To XYZ...");
        gotoXYZButton.setText("Go");
        gotoXYZButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuStageGotoXYZActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(gotoXYZButton);
        gotoXYZButton.setBounds(50, 70, 50, 30);
        
        
        probeXDecYDec.setToolTipText("Close Tweezers");
        probeXDecYDec.setText("v<");
        probeXDecYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXDecMousePressed(evt);
                probeYDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXDecYDec);
        probeXDecYDec.setBounds(220, 70, 50, 30);
        
        
        probeXIncYInc.setToolTipText("Open Tweezers");
        probeXIncYInc.setText("^>");
        probeXIncYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXIncMousePressed(evt);
                probeYIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXIncYInc);
        probeXIncYInc.setBounds(270, 20, 50, 30);
        
        
        probeXIncYDec.setToolTipText("ProbeY Down, ProbeX Right");
        probeXIncYDec.setText("v>");
        probeXIncYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXIncMousePressed(evt);
                probeYDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXIncYDec);
        probeXIncYDec.setBounds(270, 70, 50, 30);
        
        
        probeXDecYInc.setToolTipText("ProbeY Up, ProbeX Left");
        probeXDecYInc.setText("^<");
        probeXDecYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXDecMousePressed(evt);
                probeYIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXDecYInc);
        probeXDecYInc.setBounds(220, 20, 50, 30);
        
        
        gotoTweezerButton.setToolTipText("Go To Tweezer Deflection...");
        gotoTweezerButton.setText("Go");
        gotoTweezerButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuTweezersGotoActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(gotoTweezerButton);
        gotoTweezerButton.setBounds(160, 110, 50, 30);
        
        setJMenuBar(jMenuBar1);
        
        setBounds(0, 0, 335, 275);
    }//GEN-END:initComponents

  private void menuTweezersQuickCalibrateActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersQuickCalibrateActionPerformed
    // Assume tips are not touching anything.  Just and zeroize tip offsets.
    _control.resetXTipStrain();
    _control.resetYTipStrain();
  }//GEN-LAST:event_menuTweezersQuickCalibrateActionPerformed

  private void menuScriptGotoEdge1ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptGotoEdge1ActionPerformed
// Add your handling code here:
    new GotoXYZ (this, _control, _edge1.x, _edge1.y, _edge1.z).show ();
  }//GEN-LAST:event_menuScriptGotoEdge1ActionPerformed

  private void menuScriptGotoPallet1ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptGotoPallet1ActionPerformed
// Add your handling code here:
    new GotoXYZ (this, _control, _pallet1.x, _pallet1.y, _pallet1.z).show ();
  }//GEN-LAST:event_menuScriptGotoPallet1ActionPerformed

  private void menuScriptGraspBlockActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptGraspBlockActionPerformed
    new GraspBlock (this, _tools, _pallet1, _edge1).show ();
  }//GEN-LAST:event_menuScriptGraspBlockActionPerformed

  private void menuScriptSetPallet1ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptSetPallet1ActionPerformed
    _pallet1.x = _control.stageX();
    _pallet1.y = _control.stageY();
    _pallet1.z = _control.stageZ();
  }//GEN-LAST:event_menuScriptSetPallet1ActionPerformed

  private void menuScriptSetEdge1ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptSetEdge1ActionPerformed
    _edge1.x = _control.stageX();
    _edge1.y = _control.stageY();
    _edge1.z = _control.stageZ();
  }//GEN-LAST:event_menuScriptSetEdge1ActionPerformed

  private void menuTweezersDecreaseProbeYToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersDecreaseProbeYToContactActionPerformed
    try {
        _tools.moveProbeYToContact (false);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuTweezersDecreaseProbeYToContactActionPerformed

  private void menuTweezersIncreaseProbeYToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersIncreaseProbeYToContactActionPerformed
    try {
        _tools.moveProbeYToContact (true);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuTweezersIncreaseProbeYToContactActionPerformed

  private void menuTweezersDecreaseProbeXToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersDecreaseProbeXToContactActionPerformed
    try {
        _tools.moveProbeXToContact (false);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuTweezersDecreaseProbeXToContactActionPerformed

  private void menuTweezersIncreaseProbeXToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersIncreaseProbeXToContactActionPerformed
    try {
        _tools.moveProbeXToContact (true);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuTweezersIncreaseProbeXToContactActionPerformed

  private void menuStageDecreaseYToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageDecreaseYToContactActionPerformed
    try {
        // Only move a small distance because the user does not have a cancel button
        _tools.moveStageYToContact (false, 0.3);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuStageDecreaseYToContactActionPerformed

  private void menuStageIncreaseYToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageIncreaseYToContactActionPerformed
    try {
        // Only move a small distance because the user does not have a cancel button
        _tools.moveStageYToContact (true, 0.3);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuStageIncreaseYToContactActionPerformed

  private void menuStageDecreaseXToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageDecreaseXToContactActionPerformed
    try {
        // Only move a small distance because the user does not have a cancel button
        _tools.moveStageXToContact (false, 0.3);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuStageDecreaseXToContactActionPerformed

  private void menuStageIncreaseXToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageIncreaseXToContactActionPerformed
    try {
        // Only move a small distance because the user does not have a cancel button
        _tools.moveStageXToContact (true, 0.3);
    } catch (CancelException e) {
        return;
    }
  }//GEN-LAST:event_menuStageIncreaseXToContactActionPerformed

  private void menuTweezersGotoActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersGotoActionPerformed
// Add your handling code here:
    new GotoProbes (this, _tools).show ();
  }//GEN-LAST:event_menuTweezersGotoActionPerformed

  private void menuTweezersCalibrateActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersCalibrateActionPerformed
// Add your handling code here:
    new Calibrate (this, _tools).show ();
  }//GEN-LAST:event_menuTweezersCalibrateActionPerformed

  private void menuStageResetZActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageResetZActionPerformed
    // Add your handling code here:
    _control.setStageXYZ (_control.stageX(), _control.stageY(), 0.);
  }//GEN-LAST:event_menuStageResetZActionPerformed

  private void menuStageResetXYActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageResetXYActionPerformed
    // Add your handling code here:
    _control.setStageXYZ (0., 0., _control.stageZ());
  }//GEN-LAST:event_menuStageResetXYActionPerformed

  private void menuStageGotoXYZActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageGotoXYZActionPerformed
    // Add your handling code here:
    new GotoXYZ (this, _control, _control.stageX(), _control.stageY(), _control.stageZ()).show ();
  }//GEN-LAST:event_menuStageGotoXYZActionPerformed

  private void menuScripClearFramesActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScripClearFramesActionPerformed
    // Add your handling code here:
    // Maybe we should put up a confirnation dialog?
    _frames.clear();
    _frameCount.setText ("0 frames");
  }//GEN-LAST:event_menuScripClearFramesActionPerformed

  private void menuScriptRunFramesActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptRunFramesActionPerformed
    // Add your handling code here:
    new RunFrames (this, _frames, _control).show ();
  }//GEN-LAST:event_menuScriptRunFramesActionPerformed

  private void menuScriptAddFrameActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptAddFrameActionPerformed
    // Add your handling code here:
    _frames.addElement (new OrthoTweezersFrame (_control));
    _frameCount.setText ("" + _frames.size() + " frames");
  }//GEN-LAST:event_menuScriptAddFrameActionPerformed

  private void menuViewMonitorActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuViewMonitorActionPerformed
    // Add your handling code here:
    if (_monitor.isVisible())
    // The monitor dialog is already visible
    return;

    // Assume the monitor dialog is not visible because its closeDialog has already
    //   been called.  We can dereference it and create a new one.
    _monitor = new OrthoTweezersMonitor (this, _control);
    _monitor.show();
  }//GEN-LAST:event_menuViewMonitorActionPerformed

  private void motionTimerOnTime (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_motionTimerOnTime
    // Add your handling code here:
    if (_moveX  != 0) {
        _control.xStep (_moveX > 0);
    }
    if (_moveY  != 0) {
        _control.yStep (_moveY > 0);
    }
    if (_moveZ  != 0) {
        _control.zStep (_moveZ > 0);
    }

    if (_moveProbeX != 0) {
        _control.moveProbeX (_control.probeXPosition() + _moveProbeX * .001);
    }
    if (_moveProbeY != 0) {
        _control.moveProbeY (_control.probeYPosition() + _moveProbeY * .001);
    }
  }//GEN-LAST:event_motionTimerOnTime

  private void probeXIncMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeXIncMousePressed
    // Add your handling code here:
    _moveProbeX = 1;
  }//GEN-LAST:event_probeXIncMousePressed

  private void probeXDecMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeXDecMousePressed
    // Add your handling code here:
    _moveProbeX = -1;
  }//GEN-LAST:event_probeXDecMousePressed

  private void probeYDecMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeYDecMousePressed
    // Add your handling code here:
    _moveProbeY = -1;
  }//GEN-LAST:event_probeYDecMousePressed

  private void probeYIncMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeYIncMousePressed
    // Add your handling code here:
    _moveProbeY = 1;
  }//GEN-LAST:event_probeYIncMousePressed

  private void probeXYReleased (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeXYReleased
    // Add your handling code here:
    _moveProbeX = 0;
    _moveProbeY = 0;
  }//GEN-LAST:event_probeXYReleased

/** All the stage movement button Release actions call this.
 */
  private void xyzReleased (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_xyzReleased
    // Add your handling code here:
    _moveX = 0;
    _moveY = 0;
    _moveZ = 0;
  }//GEN-LAST:event_xyzReleased

  private void yDecreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_yDecreaseMousePressed
    // Add your handling code here:
    _moveY = -1;
  }//GEN-LAST:event_yDecreaseMousePressed

  private void xIncreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_xIncreaseMousePressed
    // Add your handling code here:
    _moveX = 1;
  }//GEN-LAST:event_xIncreaseMousePressed

  private void xDecreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_xDecreaseMousePressed
    // Add your handling code here:
    _moveX = -1;
  }//GEN-LAST:event_xDecreaseMousePressed

  private void yIncreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_yIncreaseMousePressed
    // Add your handling code here:
    _moveY = 1;
  }//GEN-LAST:event_yIncreaseMousePressed

  private void zDecreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_zDecreaseMousePressed
    // Add your handling code here:
    _moveZ = -1;
  }//GEN-LAST:event_zDecreaseMousePressed

  private void zIncreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_zIncreaseMousePressed
    // Add your handling code here:
    _moveZ = 1;
  }//GEN-LAST:event_zIncreaseMousePressed

/** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        if (_control.probeXPosition() != 0. || _control.probeYPosition() != 0.) {
            // _control.reset() will move the probes, so raise Z a little 
            // so that the moving probes won't hit a part.  This check
            // makes it so that if we just start and stop the application,
            // we won't necessarily raise Z a little each time.
            try {
                _status.setText ("raising Z");
                _control.moveToZ (_control.stageZ() + .2);
            } catch (CancelException e) {
                // do nothing
            }
        }

        // Return tweezers, etc. to rest state.  We must do this
        // before calling _control.getPreferences because reset() may move the stage.
        _control.reset();

        if (_preferencesFile != null) {
            // Get the parameters from the control object
            _control.getPreferences (_preferences);
            
            _preferences.putDouble (Preferences.EDGE1_X, _edge1.x);
            _preferences.putDouble (Preferences.EDGE1_Y, _edge1.y);
            _preferences.putDouble (Preferences.EDGE1_Z, _edge1.z);
            _preferences.putDouble (Preferences.PALLET1_X, _pallet1.x);
            _preferences.putDouble (Preferences.PALLET1_Y, _pallet1.y);
            _preferences.putDouble (Preferences.PALLET1_Z, _pallet1.z);
            
            _preferences.saveToFile(_preferencesFile);
        }

        System.exit (0);
    }//GEN-LAST:event_exitForm

    /**
     * @param args the command line arguments
     */
    public static void main (String args[]) {
        new OrthoTweezers ().show ();
    }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private org.netbeans.examples.lib.timerbean.Timer motionTimer;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenu menuView;
    private javax.swing.JMenuItem menuViewMonitor;
    private javax.swing.JMenu menuScript;
    private javax.swing.JMenuItem menuScriptAddFrame;
    private javax.swing.JMenuItem menuScriptRunFrames;
    private javax.swing.JMenuItem menuScripClearFrames;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JMenuItem menuScriptSetEdge1;
    private javax.swing.JMenuItem menuScriptSetPallet1;
    private javax.swing.JMenuItem menuScriptGotoEdge1;
    private javax.swing.JMenuItem menuScriptGotoPallet1;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JMenuItem menuScriptGraspBlock;
    private javax.swing.JMenu menuStage;
    private javax.swing.JMenuItem menuStageGotoXYZ;
    private javax.swing.JMenuItem menuStageResetXY;
    private javax.swing.JMenuItem menuStageResetZ;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JMenuItem menuStageIncreaseXToContact;
    private javax.swing.JMenuItem menuStageDecreaseXToContact;
    private javax.swing.JMenuItem menuStageIncreaseYToContact;
    private javax.swing.JMenuItem menuStageDecreaseYToContact;
    private javax.swing.JMenu menuTweezers;
    private javax.swing.JMenuItem menuTweezersCalibrate;
    private javax.swing.JMenuItem menuTweezersQuickCalibrate;
    private javax.swing.JMenuItem menuTweezersGoto;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JMenuItem menuTweezersIncreaseProbeXToContact;
    private javax.swing.JMenuItem menuTweezersDecreaseProbeXToContact;
    private javax.swing.JMenuItem menuTweezersIncreaseProbeYToContact;
    private javax.swing.JMenuItem menuTweezersDecreaseProbeYToContact;
    private javax.swing.JButton xDecrease;
    private javax.swing.JButton yIncrease;
    private javax.swing.JButton zDecrease;
    private javax.swing.JButton zIncrease;
    private javax.swing.JButton xIncrease;
    private javax.swing.JButton yDecrease;
    private javax.swing.JLabel _status;
    private javax.swing.JButton xIncYInc;
    private javax.swing.JButton xDecYInc;
    private javax.swing.JButton xIncYDec;
    private javax.swing.JButton xDecYDec;
    private javax.swing.JButton probeYInc;
    private javax.swing.JButton probeYDec;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JButton probeXDec;
    private javax.swing.JButton probeXInc;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JButton addFrame;
    private javax.swing.JButton runFrames;
    private javax.swing.JLabel _frameCount;
    private javax.swing.JButton gotoXYZButton;
    private javax.swing.JButton probeXDecYDec;
    private javax.swing.JButton probeXIncYInc;
    private javax.swing.JButton probeXIncYDec;
    private javax.swing.JButton probeXDecYInc;
    private javax.swing.JButton gotoTweezerButton;
    // End of variables declaration//GEN-END:variables

    javax.swing.Timer _timer;
    private OrthoTweezersControl _control;
    private OrthoTweezersTools _tools;
    
    private OrthoTweezersMonitor _monitor;
    // These are 0 for no motion, -1 for decreasing, 1 for increasing
    private int _moveX, _moveY, _moveZ;
    private int _moveProbeX, _moveProbeY;

    // Vector of OrthoTweezerFrame.
    private Vector _frames = new Vector();

    private File _preferencesFile;
    private Preferences _preferences = new Preferences ();
    
    private Point3d _edge1 = new Point3d();
    private Point3d _pallet1 = new Point3d();
}