/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved. 

Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
  
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE. 
  
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
*/
				 
package orthotweezers;

import java.util.Vector;

/** RunFrames is a modal dialog box which executes the frames in a script and
 * provides a Cancel button to interrupt it.
 *
 * @author  unknown
 */
public class RunFrames extends javax.swing.JDialog implements Runnable{

    /** Creates new modal dialog RunFrames.  After creating this, call show().
     * When the user clicks Run, a separate thread moves to each OrthoTweezersFrame
     * in frames, using control.
     */
    public RunFrames(java.awt.Frame parent, Vector frames, OrthoTweezersControl control) {
        super (parent, true);        
        _frames = frames;
        _control = control;

        initComponents ();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        _status = new javax.swing.JLabel();
        cancelButton = new javax.swing.JButton();
        runButton = new javax.swing.JButton();
        _repetitionsField = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        getContentPane().setLayout(null);
        setModal(true);
        setTitle("Run Frames");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        }
        );
        
        _status.setText("click Run to start");
        
        getContentPane().add(_status);
        _status.setBounds(10, 70, 160, 20);
        
        
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(cancelButton);
        cancelButton.setBounds(100, 40, 80, 20);
        
        
        runButton.setText("Run");
        runButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(runButton);
        runButton.setBounds(10, 40, 80, 20);
        
        
        _repetitionsField.setText("1");
        _repetitionsField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(_repetitionsField);
        _repetitionsField.setBounds(10, 10, 30, 20);
        
        
        jLabel1.setText("repetitions (-1 for infinite)");
        
        getContentPane().add(jLabel1);
        jLabel1.setBounds(50, 10, 170, 20);
        
        setBounds(0, 0, 250, 150);
    }//GEN-END:initComponents

  private void cancelButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
// Add your handling code here:
        doClose();
  }//GEN-LAST:event_cancelButtonActionPerformed

  private void runButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_runButtonActionPerformed
    // Add your handling code here:
    try {
        _repetitions = Integer.parseInt (_repetitionsField.getText());
    } catch (NumberFormatException e) {
        // Didn't type a number
        _status.setText ("Please type a valid number for repetitions");
        // Just return and allow the user to try again.
        return;
    }

    // Should only be able to run once, so disable it.
    runButton.setEnabled (false);

    // Start the thread which will call run().
    _cancelThread = new CancelThread (this);
    _cancelThread.start();    
  }//GEN-LAST:event_runButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose();
    }//GEN-LAST:event_closeDialog

    private void doClose () {
        // cancel the running thread
        if (_cancelThread != null)
            // cancel the running thread
            _cancelThread.cancel();
        
        setVisible (false);
        dispose ();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel _status;
    private javax.swing.JButton cancelButton;
    private javax.swing.JButton runButton;
    private javax.swing.JTextField _repetitionsField;
    private javax.swing.JLabel jLabel1;
    // End of variables declaration//GEN-END:variables

    /** This is run in a separate thread to execute the frames in _frames.
     * Use _repetitions for the number of times to repeat (or a negative number
     * for infinite repetitions).
     */
    public void run() {
        try {
            // Note that if _repetitions is negative, count will never reach it
            //  and therefore will loop infinite repetitions.
            for (int count = 0; count != _repetitions; ++count) {
                for (int i = 0; i < _frames.size(); ++i) {
                    CancelThread.check();

                    OrthoTweezersFrame frame = (OrthoTweezersFrame)_frames.elementAt (i);
                    _status.setText ("Repetition " + (count + 1) + ", frame " + i);
                    frame.moveTo (_control);
                }
            }
            
            _status.setText ("done");
            cancelButton.setText ("Close");
        } catch (CancelException e) {
            // user cancelled.  
            _status.setText (e.getMessage());
        }
    }
    
    private Vector _frames;
    private OrthoTweezersControl _control;
    private CancelThread _cancelThread;
    private int _repetitions;
}