/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved.
 
Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
 
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
 
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
 */

package orthotweezers;

/** An ExecuteDialog provides a dialog box with a Go and Cancel button.
 *  When Go is clicked, a new CancelThread is created and the code in
 *  the execute() method is called.  A subclass should override execute().
 *  If the Cancel button is pushed, execute() will throw a CancelException
 *  which will terminate the thread.
 *
 * @author  unknown
 */
public class ExecuteDialog extends javax.swing.JDialog implements Runnable {
    
    /** Creates new ExecuteDialog.
     *
     * @param parent    The parent Frame
     * @param title     The string to display in the dialog box title bar
     * @param goButtonText  The name to set the Go button to
     * @param initialStatus The initial string for the status label.
     * @param executeImmediately    If false, then the user must click the "go"
     *          button to begin executing.  If true, this begins executing
     *          as soon as the dialog box is visible.
     */
    public ExecuteDialog
      (java.awt.Frame parent, String title, String goButtonText, String initialStatus,
       boolean executeImmediately) {
        super (parent, true);
        _goButtonText = goButtonText;
        
        initComponents ();
        setTitle(title);
        goPauseButton.setText(_goButtonText);
        _status.setText (initialStatus);
        _executeImmediately = executeImmediately;
    }
    
    /** Creates new ExecuteDialog.  This does not execute immediately, but waits
     *     for the user to click the "go" button.
     *
     * @param parent    The parent Frame
     * @param title     The string to display in the dialog box title bar
     * @param goButtonText  The name to set the Go button to
     * @param initialStatus The initial string for the status label.
     */
    public ExecuteDialog
    (java.awt.Frame parent, String title, String goButtonText, String initialStatus) {
        this (parent, title, goButtonText, initialStatus, false);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        goPauseButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        _status = new javax.swing.JLabel();
        getContentPane().setLayout(null);
        setTitle("Executing");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowOpened(java.awt.event.WindowEvent evt) {
                formWindowOpened(evt);
            }
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        }
        );
        
        goPauseButton.setText("Go");
        goPauseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                goPauseButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(goPauseButton);
        goPauseButton.setBounds(20, 10, 120, 20);
        
        
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(cancelButton);
        cancelButton.setBounds(170, 10, 80, 20);
        
        
        _status.setText("Click Go to begin");
        
        getContentPane().add(_status);
        _status.setBounds(10, 40, 330, 20);
        
        setBounds(0, 0, 350, 100);
    }//GEN-END:initComponents

  private void formWindowOpened(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowOpened
      if (_executeImmediately)
          // goPauseButtonActionPerformed doesn't look at the evt so pass null.
          goPauseButtonActionPerformed(null);
  }//GEN-LAST:event_formWindowOpened

  private void goPauseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_goPauseButtonActionPerformed
    // Make sure the cancel button says Cancel and not Close
    cancelButton.setText ("Cancel");    

    if (_cancelThread == null) {
        // This is the first time we pushed Go
        goPauseButton.setText ("Pause");    
    
        // Start the thread which will call run().
        _cancelThread = new CancelThread (this);
        _cancelThread.start();
    }
    else {
        // Thread exists.  
        if (_cancelThread.isPaused()) {
            // Unpause and set button back to Pause to allow pausing again
            _cancelThread.unpause();
            goPauseButton.setText ("Pause");    
        }
        else {
            // Not paused.  Assume it is not cancelled since if it were, we
            //   would have exited the dialog box.
            _cancelThread.pause();
            goPauseButton.setText ("Resume");    
        }
    }
  }//GEN-LAST:event_goPauseButtonActionPerformed
      
  private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
      doClose();
  }//GEN-LAST:event_cancelButtonActionPerformed
  
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        doClose ();
    }//GEN-LAST:event_closeDialog
    
    private void doClose () {
        if (_cancelThread != null && _cancelMessage != null)
            // Usually, the dialog box should close before the user gets a chance
            //   to see this, but they will see it if the running thread does not
            //   exit immediately.
            System.out.println(_cancelMessage);
        
        if (_cancelThread != null)
            // cancel the running thread. It is okay if it is already canceled.
            _cancelThread.cancel();
        
        _cancelThread = null;
        setVisible (false);
        dispose ();
    }
    
    /** Usually when the user clicks Cancel, the executing thread will catch
     * it and exit immediately.  But if it doesn't you may want to use this to set the
     * message which will be printed to System.out while waiting for the running
     * thread to exit.  Perhaps this could be a message to tell the user something
     * to do to make sure the running thread can get to a place to call CancelThread.check().
     */
    public void setCancelMessage (String cancelMessage) {
        _cancelMessage = cancelMessage;
    }
    
   /** This is run in a separate thread and calls execute() and catches
     * a possible CancelException.
     *
     * @see #execute
     */
    public void run() {
        try {
            execute();
            
            // Re-enable the "go" button so we can go again
            goPauseButton.setText (_goButtonText);
            cancelButton.setText ("Close");
            
            // We are just about to exit this thread, so we don't need the _cancelThread
            //   which defines this thread anymore.
            _cancelThread = null;
        } catch (CancelException e) {
            // user cancelled.
            _status.setText (e.getMessage());
        }
    }
    
    /** This is called once the Go button is pressed and runs in a new thread.
     * It is expected that a subclass will override this method.
     * The user may cancel this thread by pressing the Cancel button.  Therefore,
     * this method can throw a CancelException.  This method, or the methods it
     * calls, should periodically call CancelThread.check() in order to check
     * for the cancel condition.  Note that execute() can call setStatus() to
     * display a status message in the dialog box.  When finished, execute()
     * should call something like setStatus("done").
     *
     * @throws CancelException
     * @see CancelThread#check
     */
    public void execute() throws CancelException {
    }
    
    /** Set the status label to the given text.  This can be called
     * from execute() to display information in the dialog box while running.
     */
    public void setStatus (String text) {
        _status.setText (text);
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton goPauseButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel _status;
    // End of variables declaration//GEN-END:variables
    
    private CancelThread _cancelThread;
    private String _goButtonText;
    private boolean _executeImmediately = false;
    private String _cancelMessage;
}
