/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved.
 
Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
 
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
 
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
 */

package orthotweezers;

import javax.vecmath.*;

/** GraspBlock is a modal dialog box which performs operation of grasping a block
 * and provides a Cancel button to interrupt it.
 *
 * @author  unknown
 */
public class GraspBlock extends ExecuteDialog {
    
    /** Creates new modal GraspBlock dialog box.  When the user presses
     *    the Grasp button, this runs an algorithm to grasp a block.
     *    The algorithm is run in a separate
     *    thread and the user can cancel.
     *
     * @param parent    The parent Frame
     * @param tools   The OrthoTweezersTools used to control the system.  The
     *                  underlying OrthoTweezersControl is obtained from this.
     */
    public GraspBlock(java.awt.Frame parent, OrthoTweezersTools tools) {
        super (parent, "Grasp Block", "Grasp",
        "Make sure probes are positioned and click Grasp");
        _tools = tools;
        _control = tools.control();
    }
    
    /** This is called when the new thread is created to execute the procedure.
     */
    public void execute() throws CancelException {
        _control.moveProbeX (1.);
        _control.moveProbeY (1.);
        // The probes should not be touching, so quick calibrate tip offsets
        _control.resetXTipStrain();
        _control.resetYTipStrain();
        
        setStatus ("Finding lower right edge");
        if (!_tools.sweepProbeXToContact(false, .5)) {
            // can't contact the block.  Perhaps Z is too high, or we should
            //   move over in X or farther in Y and try again.
            setStatus ("ERROR: Can't contact block lower corner");
            return;
        }
        setStatus ("Finding upper left edge");
        if (!_tools.sweepProbeYToContact(false, .5)) {
            // can't contact the block.  Perhaps Z is too high, or we should
            //   move over in Y or farther in X and try again.
            setStatus ("ERROR: Can't contact block left corner");
            return;
        }
        setStatus ("Finding corners and grasping");
        String errorString = _tools.grasp(.1, GRIP_FORCE);
        if (errorString != null) {
            setStatus (errorString);
            return;
        }
        
        // Lift and rotate to 45 degrees
        setStatus ("Lifting and rotating");
        _control.moveToZ (_control.stageZ() + .3);
        _tools.seekZAxisAngle (Math.PI / 4., GRIP_FORCE);

        setStatus ("done");
    }
    
    final static double GRIP_FORCE = 4.;
    private OrthoTweezersTools _tools;
    private OrthoTweezersControl _control;
}
