/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved.
 
Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
 
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
 
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
 */

package orthotweezers;

import java.io.*;
import java.awt.*;
import java.util.Vector;
import java.net.URL;
import javax.vecmath.*;
import java.text.NumberFormat;
import org.python.util.*;
import org.python.core.*;

/**
 * Main window for the OrthoTweezers user interface.
 *
 * @author  unknown
 */
public class OrthoTweezers extends javax.swing.JFrame {
    
    /** Creates new form OrthoTweezers.
     */
    public OrthoTweezers() {
        initComponents ();
        for (int i = 0; i < MAX_LOCATIONS; ++i)
            _locations[i] = new Point3d();
        
        _control = new OrthoTweezersControl ();
        _tools = new OrthoTweezersTools (_control);
        
        // Create and show the modeless monitor dialog
        _monitor = new OrthoTweezersMonitor (this, _tools);
        _monitor.show();
        
        // Find the filename of the preferences file.
        URL classURL = getClass().getResource("OrthoTweezers.class");
        if (classURL == null) {
            // This shouldn't really happen since the .class file should always exist
            // Debug: may need to handle case where the .class file is in a jar file
            _status.setText ("Cannot find OrthoTweezers.class file");
        }
        else {
            File classFile = new File (classURL.getPath());
            // Construct the preferences file path from the OrthoTweezers.class directory
            _preferencesFile = new File (classFile.getParent(), "orthotweezers.ini");
            
            // Only read preferences file if it exists.  If it doesn't,
            //   then it will be created by _preferences.saveToFile() upon exiting.
            if (_preferencesFile.exists()) {
                if (_preferences.loadFromFile (_preferencesFile) == false) {
                    _status.setText ("Error reading preferences file");
                }
                else {
                    // Initialize _control from Preferences.
                    _control.setFromPreferences (_preferences);
                    
                    // Get the locations
                    for (int i = 0; i < MAX_LOCATIONS; ++i) {
                        String prefix = "location" + i;

                        _locations[i].x = _preferences.maybeGetDouble (prefix + "x", 0);
                        _locations[i].y = _preferences.maybeGetDouble (prefix + "y", 0);
                        _locations[i].z = _preferences.maybeGetDouble (prefix + "z", 0);
                        _locationNames[i] = _preferences.getProperty (prefix + "name");
                    }
                    
                    _scriptDirectory = _preferences.getProperty (SCRIPT_DIRECTORY);
                }
            }
        }
        
        // Gray out menu items as needed.
        setMenuState();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        motionTimer = new org.netbeans.examples.lib.timerbean.Timer();
        jMenuBar1 = new javax.swing.JMenuBar();
        menuFile = new javax.swing.JMenu();
        menuFileNewScript = new javax.swing.JMenuItem();
        menuFileOpenScript = new javax.swing.JMenuItem();
        menuFileSaveScript = new javax.swing.JMenuItem();
        menuFileSaveScriptAs = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        menuFileExit = new javax.swing.JMenuItem();
        menuScript = new javax.swing.JMenu();
        menuScriptGraspBlock = new javax.swing.JMenuItem();
        menuScriptHeater1On = new javax.swing.JMenuItem();
        menuScriptHeater1Off = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        menuScriptSetLocation = new javax.swing.JMenu();
        menuScriptSetLocation0 = new javax.swing.JMenuItem();
        menuScriptSetLocation1 = new javax.swing.JMenuItem();
        menuScriptSetLocation2 = new javax.swing.JMenuItem();
        menuScriptGoToLocation = new javax.swing.JMenu();
        menuScriptGotoLocation0 = new javax.swing.JMenuItem();
        menuScriptGotoLocation1 = new javax.swing.JMenuItem();
        menuScriptGotoLocation2 = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JSeparator();
        menuScriptUsePythonConsole = new javax.swing.JMenuItem();
        menuScriptAddFrame = new javax.swing.JMenuItem();
        menuScriptRunScript = new javax.swing.JMenuItem();
        menuStage = new javax.swing.JMenu();
        menuStageGotoXYZ = new javax.swing.JMenuItem();
        menuStageResetXY = new javax.swing.JMenuItem();
        menuStageResetZ = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        menuStageSeekZSurface = new javax.swing.JMenuItem();
        menuTweezers = new javax.swing.JMenu();
        menuTweezersCalibrate = new javax.swing.JMenuItem();
        menuTweezersResetArmOffsets = new javax.swing.JMenuItem();
        menuTweezersResetTipOffsets = new javax.swing.JMenuItem();
        menuTweezersGoto = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        menuTweezersIncreaseProbeXToContact = new javax.swing.JMenuItem();
        menuTweezersDecreaseProbeXToContact = new javax.swing.JMenuItem();
        menuTweezersIncreaseProbeYToContact = new javax.swing.JMenuItem();
        menuTweezersDecreaseProbeYToContact = new javax.swing.JMenuItem();
        xDecrease = new javax.swing.JButton();
        yIncrease = new javax.swing.JButton();
        zDecrease = new javax.swing.JButton();
        zIncrease = new javax.swing.JButton();
        xIncrease = new javax.swing.JButton();
        yDecrease = new javax.swing.JButton();
        _status = new javax.swing.JLabel();
        xIncYInc = new javax.swing.JButton();
        xDecYInc = new javax.swing.JButton();
        xIncYDec = new javax.swing.JButton();
        xDecYDec = new javax.swing.JButton();
        probeYInc = new javax.swing.JButton();
        probeYDec = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        probeXDec = new javax.swing.JButton();
        probeXInc = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        gotoXYZButton = new javax.swing.JButton();
        probeXDecYDec = new javax.swing.JButton();
        probeXIncYInc = new javax.swing.JButton();
        probeXIncYDec = new javax.swing.JButton();
        probeXDecYInc = new javax.swing.JButton();
        gotoTweezerButton = new javax.swing.JButton();
        runScriptButton = new javax.swing.JButton();
        addFrameButton = new javax.swing.JButton();
        consoleButton = new javax.swing.JButton();
        motionTimer.setDelay(3L);
        motionTimer.addTimerListener(new org.netbeans.examples.lib.timerbean.TimerListener() {
            public void onTime(java.awt.event.ActionEvent evt) {
                motionTimerOnTime(evt);
            }
        }
        );
        
        menuFile.setText("File");
          
          menuFileNewScript.setText("New Script");
            menuFileNewScript.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuFileNewScriptActionPerformed(evt);
                }
            }
            );
            menuFile.add(menuFileNewScript);
            
          menuFileOpenScript.setText("Open Script...");
            menuFileOpenScript.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuFileOpenScriptActionPerformed(evt);
                }
            }
            );
            menuFile.add(menuFileOpenScript);
            
          menuFileSaveScript.setText("Save Script");
            menuFileSaveScript.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuFileSaveScriptActionPerformed(evt);
                }
            }
            );
            menuFile.add(menuFileSaveScript);
            
          menuFileSaveScriptAs.setText("Save Script As...");
            menuFileSaveScriptAs.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuFileSaveScriptAsActionPerformed(evt);
                }
            }
            );
            menuFile.add(menuFileSaveScriptAs);
            
          menuFile.add(jSeparator4);
            
          menuFileExit.setText("Exit");
            menuFileExit.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuFileExitActionPerformed(evt);
                }
            }
            );
            menuFile.add(menuFileExit);
            jMenuBar1.add(menuFile);
          
        menuScript.setText("Script");
          
          menuScriptGraspBlock.setText("Grasp Block");
            menuScriptGraspBlock.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptGraspBlockActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptGraspBlock);
            
          menuScriptHeater1On.setText("Heater 1 On");
            menuScriptHeater1On.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptHeater1OnActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptHeater1On);
            
          menuScriptHeater1Off.setText("Heater 1 Off");
            menuScriptHeater1Off.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptHeater1OffActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptHeater1Off);
            
          menuScript.add(jSeparator3);
            
          menuScriptSetLocation.setText("Set Location");
            
            menuScriptSetLocation0.setText("Set Location 0");
              menuScriptSetLocation0.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent evt) {
                      menuScriptSetLocation0ActionPerformed(evt);
                  }
              }
              );
              menuScriptSetLocation.add(menuScriptSetLocation0);
              
            menuScriptSetLocation1.setText("Set Location 1");
              menuScriptSetLocation1.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent evt) {
                      menuScriptSetLocation1ActionPerformed(evt);
                  }
              }
              );
              menuScriptSetLocation.add(menuScriptSetLocation1);
              
            menuScriptSetLocation2.setText("Set Location 2");
              menuScriptSetLocation2.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent evt) {
                      menuScriptSetLocation2ActionPerformed(evt);
                  }
              }
              );
              menuScriptSetLocation.add(menuScriptSetLocation2);
              menuScript.add(menuScriptSetLocation);
            
          menuScriptGoToLocation.setText("Go To Location");
            
            menuScriptGotoLocation0.setText("Go To Location 0...");
              menuScriptGotoLocation0.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent evt) {
                      menuScriptGotoLocation0ActionPerformed(evt);
                  }
              }
              );
              menuScriptGoToLocation.add(menuScriptGotoLocation0);
              
            menuScriptGotoLocation1.setText("Go To Location 1...");
              menuScriptGotoLocation1.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent evt) {
                      menuScriptGotoLocation1ActionPerformed(evt);
                  }
              }
              );
              menuScriptGoToLocation.add(menuScriptGotoLocation1);
              
            menuScriptGotoLocation2.setText("Go To Location 2...");
              menuScriptGotoLocation2.addActionListener(new java.awt.event.ActionListener() {
                  public void actionPerformed(java.awt.event.ActionEvent evt) {
                      menuScriptGotoLocation2ActionPerformed(evt);
                  }
              }
              );
              menuScriptGoToLocation.add(menuScriptGotoLocation2);
              menuScript.add(menuScriptGoToLocation);
            
          menuScript.add(jSeparator5);
            
          menuScriptUsePythonConsole.setText("Use Python Console");
            menuScriptUsePythonConsole.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptUsePythonConsoleActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptUsePythonConsole);
            
          menuScriptAddFrame.setText("Add Frame To Script");
            menuScriptAddFrame.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptAddFrameActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptAddFrame);
            
          menuScriptRunScript.setText("Run Script...");
            menuScriptRunScript.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuScriptRunScriptActionPerformed(evt);
                }
            }
            );
            menuScript.add(menuScriptRunScript);
            jMenuBar1.add(menuScript);
          
        menuStage.setText("Stage");
          
          menuStageGotoXYZ.setText("Go To XYZ...");
            menuStageGotoXYZ.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageGotoXYZActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageGotoXYZ);
            
          menuStageResetXY.setText("Reset XY");
            menuStageResetXY.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageResetXYActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageResetXY);
            
          menuStageResetZ.setText("Reset Z");
            menuStageResetZ.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageResetZActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageResetZ);
            
          menuStage.add(jSeparator2);
            
          menuStageSeekZSurface.setText("Seek Z Surface");
            menuStageSeekZSurface.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuStageSeekZSurfaceActionPerformed(evt);
                }
            }
            );
            menuStage.add(menuStageSeekZSurface);
            jMenuBar1.add(menuStage);
          
        menuTweezers.setText("Tweezers");
          
          menuTweezersCalibrate.setText("Calibrate...");
            menuTweezersCalibrate.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersCalibrateActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersCalibrate);
            
          menuTweezersResetArmOffsets.setText("Reset Arm Offsets");
            menuTweezersResetArmOffsets.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersResetArmOffsetsActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersResetArmOffsets);
            
          menuTweezersResetTipOffsets.setText("Reset Tip Offsets");
            menuTweezersResetTipOffsets.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersResetTipOffsetsActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersResetTipOffsets);
            
          menuTweezersGoto.setText("Goto Deflection...");
            menuTweezersGoto.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersGotoActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersGoto);
            
          menuTweezers.add(jSeparator1);
            
          menuTweezersIncreaseProbeXToContact.setText("Increase Probe X To Contact");
            menuTweezersIncreaseProbeXToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersIncreaseProbeXToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersIncreaseProbeXToContact);
            
          menuTweezersDecreaseProbeXToContact.setText("Decrease Probe X To Contact");
            menuTweezersDecreaseProbeXToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersDecreaseProbeXToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersDecreaseProbeXToContact);
            
          menuTweezersIncreaseProbeYToContact.setText("Increase Probe Y To Contact");
            menuTweezersIncreaseProbeYToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersIncreaseProbeYToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersIncreaseProbeYToContact);
            
          menuTweezersDecreaseProbeYToContact.setText("DecreaseProbeYToContact");
            menuTweezersDecreaseProbeYToContact.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    menuTweezersDecreaseProbeYToContactActionPerformed(evt);
                }
            }
            );
            menuTweezers.add(menuTweezersDecreaseProbeYToContact);
            jMenuBar1.add(menuTweezers);
          getContentPane().setLayout(null);
        setTitle("Ortho Tweezers");
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        }
        );
        
        xDecrease.setText("<");
        xDecrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xDecrease);
        xDecrease.setLocation(10, 70);
        xDecrease.setSize(xDecrease.getPreferredSize());
        
        
        yIncrease.setText("^");
        yIncrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                yIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(yIncrease);
        yIncrease.setLocation(50, 40);
        yIncrease.setSize(yIncrease.getPreferredSize());
        
        
        zDecrease.setText("Lower");
        zDecrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                zDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(zDecrease);
        zDecrease.setLocation(80, 10);
        zDecrease.setSize(zDecrease.getPreferredSize());
        
        
        zIncrease.setText("Raise");
        zIncrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                zIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(zIncrease);
        zIncrease.setLocation(10, 10);
        zIncrease.setSize(zIncrease.getPreferredSize());
        
        
        xIncrease.setText(">");
        xIncrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xIncrease);
        xIncrease.setLocation(100, 70);
        xIncrease.setSize(xIncrease.getPreferredSize());
        
        
        yDecrease.setText("v");
        yDecrease.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                yDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(yDecrease);
        yDecrease.setLocation(50, 100);
        yDecrease.setSize(yDecrease.getPreferredSize());
        
        
        _status.setText("initialized");
        
        getContentPane().add(_status);
        _status.setBounds(30, 180, 290, 20);
        
        
        xIncYInc.setText("/");
        xIncYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xIncreaseMousePressed(evt);
                yIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xIncYInc);
        xIncYInc.setLocation(100, 40);
        xIncYInc.setSize(xIncYInc.getPreferredSize());
        
        
        xDecYInc.setText("\\");
        xDecYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xDecreaseMousePressed(evt);
                yIncreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xDecYInc);
        xDecYInc.setLocation(10, 40);
        xDecYInc.setSize(xDecYInc.getPreferredSize());
        
        
        xIncYDec.setText("\\");
        xIncYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xIncreaseMousePressed(evt);
                yDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xIncYDec);
        xIncYDec.setLocation(100, 100);
        xIncYDec.setSize(xIncYDec.getPreferredSize());
        
        
        xDecYDec.setText("/");
        xDecYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                xDecreaseMousePressed(evt);
                yDecreaseMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                xyzReleased(evt);
            }
        }
        );
        
        getContentPane().add(xDecYDec);
        xDecYDec.setLocation(10, 100);
        xDecYDec.setSize(xDecYDec.getPreferredSize());
        
        
        probeYInc.setText("^");
        probeYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeYIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeYInc);
        probeYInc.setBounds(160, 10, 50, 30);
        
        
        probeYDec.setText("v");
        probeYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeYDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeYDec);
        probeYDec.setBounds(160, 60, 50, 30);
        
        
        jLabel2.setText("probe Y");
        
        getContentPane().add(jLabel2);
        jLabel2.setLocation(170, 40);
        jLabel2.setSize(jLabel2.getPreferredSize());
        
        
        probeXDec.setText("<");
        probeXDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXDec);
        probeXDec.setBounds(220, 100, 50, 30);
        
        
        probeXInc.setText(">");
        probeXInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXInc);
        probeXInc.setBounds(270, 100, 50, 30);
        
        
        jLabel3.setText("probe X");
        
        getContentPane().add(jLabel3);
        jLabel3.setLocation(250, 130);
        jLabel3.setSize(jLabel3.getPreferredSize());
        
        
        gotoXYZButton.setToolTipText("Go To XYZ...");
        gotoXYZButton.setText("Go");
        gotoXYZButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuStageGotoXYZActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(gotoXYZButton);
        gotoXYZButton.setBounds(50, 70, 50, 30);
        
        
        probeXDecYDec.setToolTipText("Close Tweezers");
        probeXDecYDec.setText("v<");
        probeXDecYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXDecMousePressed(evt);
                probeYDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXDecYDec);
        probeXDecYDec.setBounds(220, 60, 50, 30);
        
        
        probeXIncYInc.setToolTipText("Open Tweezers");
        probeXIncYInc.setText("^>");
        probeXIncYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXIncMousePressed(evt);
                probeYIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXIncYInc);
        probeXIncYInc.setBounds(270, 10, 50, 30);
        
        
        probeXIncYDec.setToolTipText("ProbeY Down, ProbeX Right");
        probeXIncYDec.setText("v>");
        probeXIncYDec.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXIncMousePressed(evt);
                probeYDecMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXIncYDec);
        probeXIncYDec.setBounds(270, 60, 50, 30);
        
        
        probeXDecYInc.setToolTipText("ProbeY Up, ProbeX Left");
        probeXDecYInc.setText("^<");
        probeXDecYInc.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                probeXDecMousePressed(evt);
                probeYIncMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                probeXYReleased(evt);
            }
        }
        );
        
        getContentPane().add(probeXDecYInc);
        probeXDecYInc.setBounds(220, 10, 50, 30);
        
        
        gotoTweezerButton.setToolTipText("Go To Tweezer Deflection...");
        gotoTweezerButton.setText("Go");
        gotoTweezerButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuTweezersGotoActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(gotoTweezerButton);
        gotoTweezerButton.setBounds(160, 100, 50, 30);
        
        
        runScriptButton.setText("Run Script...");
        runScriptButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuScriptRunScriptActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(runScriptButton);
        runScriptButton.setBounds(120, 150, 110, 20);
        
        
        addFrameButton.setText("Add Frame");
        addFrameButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuScriptAddFrameActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(addFrameButton);
        addFrameButton.setBounds(10, 150, 100, 20);
        
        
        consoleButton.setText("Console");
        consoleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuScriptUsePythonConsoleActionPerformed(evt);
            }
        }
        );
        
        getContentPane().add(consoleButton);
        consoleButton.setBounds(240, 150, 80, 20);
        
        setJMenuBar(jMenuBar1);
        
        setBounds(0, 0, 350, 250);
    }//GEN-END:initComponents

  private void menuScriptHeater1OffActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptHeater1OffActionPerformed
    _tools.heaterOff (1);
  }//GEN-LAST:event_menuScriptHeater1OffActionPerformed

  private void menuScriptHeater1OnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptHeater1OnActionPerformed
    _tools.heaterOn(1);
  }//GEN-LAST:event_menuScriptHeater1OnActionPerformed

  private void menuStageSeekZSurfaceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageSeekZSurfaceActionPerformed
    new SeekZSurface (this, _tools).show();
  }//GEN-LAST:event_menuStageSeekZSurfaceActionPerformed

  private void menuTweezersResetArmOffsetsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersResetArmOffsetsActionPerformed
      _control.resetXArmStrain();
      _control.resetYArmStrain();
  }//GEN-LAST:event_menuTweezersResetArmOffsetsActionPerformed

  private void menuTweezersResetTipOffsetsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersResetTipOffsetsActionPerformed
      // Assume tips are not touching anything.  Just and zeroize tip offsets.
      _tools.resetTipStrains();
  }//GEN-LAST:event_menuTweezersResetTipOffsetsActionPerformed

  private void menuScriptGotoLocation2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptGotoLocation2ActionPerformed
    menuGotoLocation (2);
  }//GEN-LAST:event_menuScriptGotoLocation2ActionPerformed

  private void menuScriptGotoLocation1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptGotoLocation1ActionPerformed
    menuGotoLocation (1);
  }//GEN-LAST:event_menuScriptGotoLocation1ActionPerformed

  private void menuScriptGotoLocation0ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptGotoLocation0ActionPerformed
    menuGotoLocation (0);
  }//GEN-LAST:event_menuScriptGotoLocation0ActionPerformed

  private void menuGotoLocation (int i) {
      new GotoXYZ (this, _tools, _locations[i].x, _locations[i].y, _locations[i].z).show ();
  }
  
  private void menuScriptSetLocation2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptSetLocation2ActionPerformed
    _tools.getStageXYZ (_locations[2]);
  }//GEN-LAST:event_menuScriptSetLocation2ActionPerformed

  private void menuScriptSetLocation1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptSetLocation1ActionPerformed
    _tools.getStageXYZ (_locations[1]);
  }//GEN-LAST:event_menuScriptSetLocation1ActionPerformed

  private void menuScriptSetLocation0ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptSetLocation0ActionPerformed
    _tools.getStageXYZ (_locations[0]);
  }//GEN-LAST:event_menuScriptSetLocation0ActionPerformed

  private void menuScriptAddFrameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptAddFrameActionPerformed
    // Make sure there is a script
    if (_scriptEditor == null)
        // This does not use the event, so pass null
        menuFileNewScriptActionPerformed(null);
    
    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumFractionDigits (4);
    nf.setMaximumFractionDigits (4);

    _scriptEditor.textArea().append
      ("moveToXYZ (" + nf.format(_control.stageX()) + ", " + 
       nf.format(_control.stageY()) + ", " + nf.format(_control.stageZ()) + ")\n");
    _scriptEditor.textArea().append
       ("seekXYArmDeflection (" + nf.format(_control.xArmDeflection()) + ", " +
         nf.format(_control.yArmDeflection()) + ")\n");
  }//GEN-LAST:event_menuScriptAddFrameActionPerformed
    
  private void menuFileSaveScriptAsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuFileSaveScriptAsActionPerformed
      if (_scriptEditor == null)
          // There is no script to save
          return;
      
      FileDialog dialog = new FileDialog (this, "Save File As", FileDialog.SAVE);
      dialog.setDirectory (_scriptDirectory);
      dialog.setFile (_scriptFilename);
      dialog.setVisible (true);
      
      if (dialog.getFile() == null)
          // cancelled
          return;
      
      _scriptDirectory = dialog.getDirectory();
      _scriptFilename = dialog.getFile();
      _scriptEditor.setTitle ("Script - " + _scriptFilename);
      saveScript ();
  }//GEN-LAST:event_menuFileSaveScriptAsActionPerformed
  
  private void menuFileSaveScriptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuFileSaveScriptActionPerformed
      if (_scriptEditor == null)
          // There is no script to save
          return;
      
      if (_scriptFilename == null)
          // This is a new file that doesn't have a name yet.
          menuFileSaveScriptAsActionPerformed(null);
      else
          saveScript();
  }//GEN-LAST:event_menuFileSaveScriptActionPerformed
  
  private void menuFileOpenScriptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuFileOpenScriptActionPerformed
      // This will return true when _scriptEditor is null.
      if (!canClose())
          return;
      
      FileDialog dialog = new FileDialog (this, "Open Script", FileDialog.LOAD);
      dialog.setDirectory (_scriptDirectory);
      dialog.setFile ("*.py");
      dialog.setVisible (true);
      
      if (dialog.getFile() == null)
          // cancelled
          return;
      
      _scriptDirectory = dialog.getDirectory();
      _scriptFilename = dialog.getFile();
      
      // Make sure there is a _scriptEditor object and read the file into it.
      establishScriptEditor();
      loadScript();    
  }//GEN-LAST:event_menuFileOpenScriptActionPerformed
  
  /** If _scriptEditor or _scriptDirectory or _scriptFilename are null, this
   * does nothing.  Otherwise, open the filename and load the script.
   */
  private void loadScript() {
      if (_scriptEditor == null || _scriptDirectory == null || _scriptFilename == null)
          return;

      try {
          File file = new File(_scriptDirectory + _scriptFilename);
          BufferedReader reader = new BufferedReader (new FileReader (file));
          
          _scriptEditor.textArea().read(reader, _scriptFilename);
          _scriptEditor.clearModified();
          _scriptEditor.setTitle ("Script - " + _scriptFilename);
          setMenuState();
          
          reader.close ();
          _scriptLastModified = file.lastModified();
      } catch (FileNotFoundException e) {
          // This shouldn't really happen
          return;
      } catch (IOException e) {
          System.out.println("Error loading script: " + e);
      }
  }
  
  /** If there is already a _scriptEditor, this does nothing.
   * Otherwise, set up the _scriptEditor with the listener and shows it.
   */
  private void establishScriptEditor() {
      if (_scriptEditor != null)
          return;
      
      _scriptEditor = new ScriptEditor(this);
      _scriptEditor.setTitle ("Script - Untitled");
      _scriptEditor.show();
  }
  
  /** If there is no _scriptEditor, or if it has not been changed, then
   * return true.  Otherwise, put up a dialog box asking if the user wants
   * to save the file.  If the user does not want to save or if the file is
   * successfully saved, return true. Otherwise the user wants to cancel,
   * so return false.
   */
  private boolean canClose()	{
      if (_scriptEditor == null || !_scriptEditor.isModified())
          // No script or not changed, can close
          return true;
      
      Toolkit.getDefaultToolkit().beep();
      
      YesNoCancelDialog dialog = new YesNoCancelDialog
      (this, "Question", "The script has changed. Do you want to save?");
      dialog.setVisible (true);
      
      if (dialog.result() == dialog.CANCEL)
          // Choosing Cancel means to abort the close.
          return false;
      else if (dialog.result() == dialog.YES) {
          // Choosing Yes means to save the script
          menuFileSaveScriptActionPerformed(null);
          // still close, even if didn't save
          return true;
      }
      else
          // No, don't save
          return true;
  }
  
  /** If _scriptEditor or _scriptDirectory or _scriptFilename are null, this
   * does nothing.  Otherwise, open the filename and save the script.
   */
  private void saveScript() {
      if (_scriptEditor == null || _scriptDirectory == null || _scriptFilename == null)
          return;
      
      try {
          File file = new File (_scriptDirectory + _scriptFilename);
          BufferedWriter writer = new BufferedWriter (new FileWriter (file));
          
          _scriptEditor.textArea().write(writer);
          writer.close();
          _scriptLastModified = file.lastModified();
          _scriptEditor.clearModified();
      } catch (IOException e) {
          System.out.println("Error saving script: " + e);
      }
  }

  /** This is called by _scriptEditor.windowClosing().
   */
  public void scriptWindowClosing() {
        if (canClose()) {
            _scriptEditor.setVisible (false);
            _scriptEditor.dispose();
            _scriptEditor = null;
            setMenuState();
        }
  }
  
  /** Compare _scriptLastModified to the real file's modified time, and if
   * the real time is later, give the user a chance to reload from disk.
   * The real file is given by _scriptDirectory and _scriptFilename.
   * If either of these is null, this does nothing.
   */
  public void checkScriptChangeOnDisk() {  
      if (_scriptDirectory == null || _scriptFilename == null)
          return;
      
      File file = new File (_scriptDirectory + _scriptFilename);
      if (file.lastModified() <= _scriptLastModified)
          // File has not been changed on disk
          return;
      
      YesNoCancelDialog dialog = new YesNoCancelDialog
        (this, "Question", "The script has been changed on disk. Do you want to reload?");
      dialog.setVisible (true);
      
      if (dialog.result() == dialog.YES)
          loadScript();
  }
  
    /** This sets the enabled state of menu items based on the state of the system.
     * This should be called after doing something that may change the menu state
     * such as closing a file.  Note: there might be a better way where the menu
     * item figures out its own enabled state just before it is shown, but I can't
     * figure it out.
     */
  private void setMenuState() {
      menuScriptRunScript.setEnabled (_scriptEditor != null);
      runScriptButton.setEnabled (_scriptEditor != null);
      menuFileSaveScript.setEnabled (_scriptEditor != null);
      menuFileSaveScriptAs.setEnabled (_scriptEditor != null);
      
      for (int i = 0; i < 3; ++i) {
          String setString = "Set Location " + i;
          String gotoString = "Go To Location " + i;
          
          if (_locationNames[i] != null && !_locationNames[i].equals("")) {
              setString += " - " + _locationNames[i];
              gotoString += " - " + _locationNames[i];
          }
          
          gotoString += "...";
          if (i == 0) {
              menuScriptSetLocation0.setText (setString);
              menuScriptGotoLocation0.setText (gotoString);
          } else if (i == 1) {
              menuScriptSetLocation1.setText (setString);
              menuScriptGotoLocation1.setText (gotoString);
          } else if (i == 2) {
              menuScriptSetLocation2.setText (setString);
              menuScriptGotoLocation2.setText (gotoString);
          }
      }
  }
  
  private void menuFileExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuFileExitActionPerformed
      // exitForm doesn't use the evt so pass null
      exitForm(null);
  }//GEN-LAST:event_menuFileExitActionPerformed
  
  private void menuScriptUsePythonConsoleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptUsePythonConsoleActionPerformed
      establishConsole();
      new UsePythonConsole (this, _console).show();
      setMenuState();      
  }//GEN-LAST:event_menuScriptUsePythonConsoleActionPerformed
  
    /** Exit the application
     */
  private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
      if (!canClose())
          // User doesn't want to close.  This works in conjunction with
          // setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE).
          return;
      
      // Close a capture file if it is open.
      _monitor.closeCapture();
      
      if (_control.probeXPosition() != 0. || _control.probeYPosition() != 0.) {
          // _control.reset() will move the probes, so raise Z a little
          // so that the moving probes won't hit a part.  This check
          // makes it so that if we just start and stop the application,
          // we won't necessarily raise Z a little each time.
          try {
              _status.setText ("raising Z");
              _control.moveToZ (_control.stageZ() + .2);
          } catch (CancelException e) {
              // do nothing
          }
      }
      
      // Return tweezers, etc. to rest state.  We must do this
      // before calling _control.getPreferences because reset() may move the stage.
      try {
        _control.reset();
      } catch (CancelException e) {
          // ignore since we are trying to exit anyway
      }
      
      if (_preferencesFile != null) {
          // Get the parameters from the control object
          _control.getPreferences (_preferences);
          
          // Set the locations
          for (int i = 0; i < MAX_LOCATIONS; ++i) {
              String prefix = "location" + i;
              
              _preferences.putDouble (prefix + "x", _locations[i].x);
              _preferences.putDouble (prefix + "y", _locations[i].y);
              _preferences.putDouble (prefix + "z", _locations[i].z);
              _preferences.setProperty (prefix + "name", _locationNames[i]);
          }
          
          if (_scriptDirectory != null)
              _preferences.setProperty (SCRIPT_DIRECTORY, _scriptDirectory);
          
          _preferences.saveToFile(_preferencesFile);
      }
      
      System.exit (0);
  }//GEN-LAST:event_exitForm
  
  private void menuFileNewScriptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuFileNewScriptActionPerformed
      if (_scriptEditor != null) {
          // There is already a script.  Make sure we can wipe it.
          if (!canClose())
              return;
      }
      else
          establishScriptEditor();
      
      _scriptEditor.textArea().setText("");
      _scriptEditor.setTitle("Script - Untitled");
      _scriptEditor.clearModified();
      _scriptEditor.show();
      setMenuState();
      _scriptFilename = null;
      _scriptLastModified = 0L;
  }//GEN-LAST:event_menuFileNewScriptActionPerformed
  
  private void menuScriptRunScriptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptRunScriptActionPerformed
      if (_scriptEditor == null)
          // No script, so just return.  Since we gray out the menu item for
          //  Run Script in this case, it shouldn't be a problem.
          return;
      
      checkScriptChangeOnDisk();
      establishConsole();
      new RunScript (this, _console, _scriptEditor.textArea().getText()).show();
      setMenuState();
  }//GEN-LAST:event_menuScriptRunScriptActionPerformed
            
  private void menuScriptGraspBlockActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuScriptGraspBlockActionPerformed
      new GraspBlock (this, _tools).show ();
  }//GEN-LAST:event_menuScriptGraspBlockActionPerformed
      
  private void menuTweezersDecreaseProbeYToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersDecreaseProbeYToContactActionPerformed
      try {
          _tools.moveProbeYToContact (false);
      } catch (CancelException e) {
          return;
      }
  }//GEN-LAST:event_menuTweezersDecreaseProbeYToContactActionPerformed
  
  private void menuTweezersIncreaseProbeYToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersIncreaseProbeYToContactActionPerformed
      try {
          _tools.moveProbeYToContact (true);
      } catch (CancelException e) {
          return;
      }
  }//GEN-LAST:event_menuTweezersIncreaseProbeYToContactActionPerformed
  
  private void menuTweezersDecreaseProbeXToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersDecreaseProbeXToContactActionPerformed
      try {
          _tools.moveProbeXToContact (false);
      } catch (CancelException e) {
          return;
      }
  }//GEN-LAST:event_menuTweezersDecreaseProbeXToContactActionPerformed
  
  private void menuTweezersIncreaseProbeXToContactActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersIncreaseProbeXToContactActionPerformed
      try {
          _tools.moveProbeXToContact (true);
      } catch (CancelException e) {
          return;
      }
  }//GEN-LAST:event_menuTweezersIncreaseProbeXToContactActionPerformed
          
  private void menuTweezersGotoActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersGotoActionPerformed
      new GotoProbes (this, _tools).show ();
  }//GEN-LAST:event_menuTweezersGotoActionPerformed
  
  private void menuTweezersCalibrateActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuTweezersCalibrateActionPerformed
      new Calibrate (this, _tools).show ();
  }//GEN-LAST:event_menuTweezersCalibrateActionPerformed
  
  private void menuStageResetZActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageResetZActionPerformed
      _control.setStageXYZ (_control.stageX(), _control.stageY(), 0.);
  }//GEN-LAST:event_menuStageResetZActionPerformed
  
  private void menuStageResetXYActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageResetXYActionPerformed
      _control.setStageXYZ (0., 0., _control.stageZ());
  }//GEN-LAST:event_menuStageResetXYActionPerformed
  
  private void menuStageGotoXYZActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_menuStageGotoXYZActionPerformed
      new GotoXYZ (this, _tools, _control.stageX(), _control.stageY(), _control.stageZ()).show ();
  }//GEN-LAST:event_menuStageGotoXYZActionPerformed
  
  private void motionTimerOnTime (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_motionTimerOnTime
      if (_moveX  != 0) {
          _control.xStep (_moveX > 0);
      }
      if (_moveY  != 0) {
          _control.yStep (_moveY > 0);
      }
      if (_moveZ  != 0) {
          _control.zStep (_moveZ > 0);
      }
      
      try {
          if (_moveProbeX != 0) {
              _control.moveProbeX (_control.probeXPosition() + _moveProbeX * .001);
          }
          if (_moveProbeY != 0) {
              _control.moveProbeY (_control.probeYPosition() + _moveProbeY * .001);
          }
      } catch (CancelException e) {
          // Ignore the CancelException for such as small change in position
      }
  }//GEN-LAST:event_motionTimerOnTime
  
  private void probeXIncMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeXIncMousePressed
      _moveProbeX = 1;
  }//GEN-LAST:event_probeXIncMousePressed
  
  private void probeXDecMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeXDecMousePressed
      _moveProbeX = -1;
  }//GEN-LAST:event_probeXDecMousePressed
  
  private void probeYDecMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeYDecMousePressed
      _moveProbeY = -1;
  }//GEN-LAST:event_probeYDecMousePressed
  
  private void probeYIncMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeYIncMousePressed
      _moveProbeY = 1;
  }//GEN-LAST:event_probeYIncMousePressed
  
  private void probeXYReleased (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_probeXYReleased
      _moveProbeX = 0;
      _moveProbeY = 0;
  }//GEN-LAST:event_probeXYReleased
  
/** All the stage movement button Release actions call this.
 */
  private void xyzReleased (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_xyzReleased
      _moveX = 0;
      _moveY = 0;
      _moveZ = 0;
  }//GEN-LAST:event_xyzReleased
  
  private void yDecreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_yDecreaseMousePressed
      _moveY = -1;
  }//GEN-LAST:event_yDecreaseMousePressed
  
  private void xIncreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_xIncreaseMousePressed
      _moveX = 1;
  }//GEN-LAST:event_xIncreaseMousePressed
  
  private void xDecreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_xDecreaseMousePressed
      _moveX = -1;
  }//GEN-LAST:event_xDecreaseMousePressed
  
  private void yIncreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_yIncreaseMousePressed
      _moveY = 1;
  }//GEN-LAST:event_yIncreaseMousePressed
  
  private void zDecreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_zDecreaseMousePressed
      _moveZ = -1;
  }//GEN-LAST:event_zDecreaseMousePressed
  
  private void zIncreaseMousePressed (java.awt.event.MouseEvent evt) {//GEN-FIRST:event_zIncreaseMousePressed
      _moveZ = 1;
  }//GEN-LAST:event_zIncreaseMousePressed
  
    /** If _console is not null, then this does nothing.  Otherwise this
     * sets _console to a new CancelInteractiveConsole.
     * This sets the Python variables control and tools to _control and _tools
     * for access in Python scripts.
     * It also sets up easy access functions so that the Python script
     * can just call, for example, moveToXY instead of control.moveToXY.
     */
  private void establishConsole() {
      if (_console != null)
          // The _console is already established.
          return;
      
      // Create the Python interactive console.  This may run a bunch of startup
      //   routines so put up a message.
      System.out.print("Starting Python...");
      _console = new CancelInteractiveConsole();
      
      _console.exec ("from math import *");
      _console.exec ("from javax.vecmath import *");
      
      // Set up the variables for access from Python.
      _console.set ("control", _control);
      _console.set ("tools", _tools);
      // Don't advertise _monitor_.  Only advertise methods like openCapture()
      _console.set ("_monitor_", _monitor);
      // Don't advertise _orthoTweezers_.  Only advertise methods like location(i)
      _console.set ("_orthoTweezers_", this);
      
      // Set up easy access functions for monitor
      _console.exec ("def openCapture(filename,append):\n _monitor_.openCapture(filename,append)");
      _console.exec ("def startCapture(period):\n _monitor_.startCapture(period)");
      _console.exec ("def captureComment(comment):\n _monitor_.captureComment(comment)");
      _console.exec ("def stopCapture():\n _monitor_.stopCapture()");
      _console.exec ("def closeCapture():\n _monitor_.closeCapture()");

      // Set up access functions to orthoTweezers
      _console.exec ("def location(i):\n return _orthoTweezers_.location(i)");
      _console.exec ("MAX_LOCATIONS = " + MAX_LOCATIONS);
      _console.set ("locationNames", _locationNames);
      
      // Set up easy access functions for control
      _console.exec ("def moveProbeX(position):\n control.moveProbeX(position)");
      _console.exec ("def moveProbeY(position):\n control.moveProbeY(position)");
      _console.exec ("def moveToXY(x,y):\n control.moveToXY(x,y)");
      _console.exec ("def moveToZ(z):\n control.moveToZ(z)");
      _console.exec ("def probeXPosition():\n return control.probeXPosition()");
      _console.exec ("def probeYPosition():\n return control.probeYPosition()");
      _console.exec ("def stageX():\n return control.stageX()");
      _console.exec ("def stageY():\n return control.stageY()");
      _console.exec ("def stageZ():\n return control.stageZ()");
      _console.exec ("def strainErrorXArm():\n return control.strainErrorXArm()");
      _console.exec ("def strainErrorXTip():\n return control.strainErrorXTip()");
      _console.exec ("def strainErrorXArm():\n return control.strainErrorXArm()");
      _console.exec ("def strainErrorXTip():\n return control.strainErrorXTip()");
      _console.exec ("def strainOffsetXArm():\n return control.strainOffsetXArm()");
      _console.exec ("def strainOffsetXTip():\n return control.strainOffsetXTip()");
      _console.exec ("def strainOffsetXArm():\n return control.strainOffsetXArm()");
      _console.exec ("def strainOffsetXTip():\n return control.strainOffsetXTip()");
      _console.exec ("def strainToDeflectionXArm():\n return control.strainToDeflectionXArm()");
      _console.exec ("def strainToDeflectionXTip():\n return control.strainToDeflectionXTip()");
      _console.exec ("def strainToDeflectionXArm():\n return control.strainToDeflectionXArm()");
      _console.exec ("def strainToDeflectionXTip():\n return control.strainToDeflectionXTip()");
      _console.exec ("def strainToForceXTip():\n return control.strainToForceXTip()");
      _console.exec ("def strainToForceYTip():\n return control.strainToForceYTip()");
      _console.exec ("def xArmDeflection():\n return control.xArmDeflection()");
      _console.exec ("def xArmStrain():\n return control.xArmStrain()");
      _console.exec ("def xStep():\n control.xStep()");
      _console.exec ("def xTipDeflection():\n return control.xTipDeflection()");
      _console.exec ("def xTipForce():\n return control.xTipForce()");
      _console.exec ("def xTipStrain():\n return control.xTipStrain()");
      _console.exec ("def yArmDeflection():\n return control.yArmDeflection()");
      _console.exec ("def yArmStrain():\n return control.yArmStrain()");
      _console.exec ("def yStep():\n control.yStep()");
      _console.exec ("def yTipDeflection():\n return control.yTipDeflection()");
      _console.exec ("def yTipForce():\n return control.yTipForce()");
      _console.exec ("def yTipStrain():\n return control.yTipStrain()");
      _console.exec ("def zStep():\n control.zStep()");
      
      // Set up easy access functions for tools
      _console.exec ("def getStageXYZ(point):\n tools.getStageXYZ(point)");
      _console.exec ("def grasp(cornerOffset, gripForce):\n return tools.grasp(cornerOffset, gripForce)");
      _console.exec ("def heaterOff(heaterNumber):\n tools.heaterOff(heaterNumber)");
      _console.exec ("def heaterOn(heaterNumber):\n tools.heaterOn(heaterNumber)");
      _console.exec ("def heaterState(heaterNumber):\n return tools.heaterState(heaterNumber)");
      _console.exec ("def moveDeltaXY(x,y):\n tools.moveDeltaXY(x,y)");
      _console.exec ("def moveDeltaZ(z):\n tools.moveDeltaZ(z)");
      _console.exec ("def moveProbeDeltaX(x):\n tools.moveProbeDeltaX(x)");
      _console.exec ("def moveProbeDeltaY(y):\n tools.moveProbeDeltaY(y)");
      _console.exec ("def moveProbeXToContact(increasing):\n return tools.moveProbeXToContact(increasing)");
      _console.exec ("def moveProbeXToContactBounded(increasing, minDeflection,maxDeflection):\n return tools.moveProbeXToContactBounded(increasing, minDeflection,maxDeflection)");
      _console.exec ("def moveProbeYToContact(increasing):\n return tools.moveProbeYToContact(increasing)");
      _console.exec ("def moveProbeYToContactBounded(increasing, minDeflection,maxDeflection):\n return tools.moveProbeYToContactBounded(increasing, minDeflection,maxDeflection)");
      _console.exec ("def moveStageXToContact(increasing,limit):\n return tools.moveStageXToContact(increasing,limit)");
      _console.exec ("def moveStageYToContact(increasing,limit):\n return tools.moveStageYToContact(increasing,limit)");
      _console.exec ("def moveStageZToContact(limit):\n return tools.moveStageZToContact(limit)");
      _console.exec ("def moveToPoint(point):\n tools.moveToPoint(point)");
      _console.exec ("def moveToXYZ(x,y,z):\n tools.moveToXYZ(x,y,z)");
      _console.exec ("def resetTipStrains():\n tools.resetTipStrains()");
      _console.exec ("def seekXArmDeflection(deflection):\n tools.seekXArmDeflection(deflection)");
      _console.exec ("def seekXArmDeltaDeflection(deflection):\n tools.seekXArmDeltaDeflection(deflection)");
      _console.exec ("def seekXTipForce(force):\n return tools.seekXTipForce(force)");
      _console.exec ("def seekXTipForceBounded (force, minDeflection, maxDeflection):\n return tools.seekXTipForceBounded (force, minDeflection, maxDeflection)");
      _console.exec ("def seekXYArmDeflection(xDeflection,yDeflection):\n tools.seekXYArmDeflection(xDeflection,yDeflection)");
      _console.exec ("def seekYArmDeflection(deflection):\n tools.seekYArmDeflection(deflection)");
      _console.exec ("def seekYArmDeltaDeflection(deflection):\n tools.seekYArmDeltaDeflection(deflection)");
      _console.exec ("def seekYTipForce(force):\n return tools.seekYTipForce(force)");
      _console.exec ("def seekYTipForceBounded (force, minDeflection, maxDeflection):\n return tools.seekYTipForceBounded (force, minDeflection, maxDeflection)");
      _console.exec ("def seekZAxisAngle(angle, gripForce):\n return tools.seekZAxisAngle(angle, gripForce)");
      _console.exec ("def seekZSurface(limit):\n return tools.seekZSurface(limit)");
      _console.exec ("def stagePoint():\n return tools.stagePoint()");
      _console.exec ("def sweepProbeXToContact(increasing, maxDeltaY):\n return tools.sweepProbeXToContact(increasing, maxDeltaY)");
      _console.exec ("def sweepProbeYToContact(increasing, maxDeltaY):\n return tools.sweepProbeYToContact(increasing, maxDeltaY)");
      _console.exec ("def ungrasp():\n tools.ungrasp()");
      _console.exec ("def zAxisAngle():\n return tools.zAxisAngle()");
      
      System.out.println("done");
  }
  
  /** Return the location object for the given index, or null if i is
   * >= MAX_LOCATIONS.
   */
  public Point3d location(int i) {
      if (i >= MAX_LOCATIONS)
          return null;
      return _locations[i];
  }
  
    /**
     * @param args the command line arguments
     */
  public static void main (String args[]) {
      new OrthoTweezers ().show ();
  }
  
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private org.netbeans.examples.lib.timerbean.Timer motionTimer;
  private javax.swing.JMenuBar jMenuBar1;
  private javax.swing.JMenu menuFile;
  private javax.swing.JMenuItem menuFileNewScript;
  private javax.swing.JMenuItem menuFileOpenScript;
  private javax.swing.JMenuItem menuFileSaveScript;
  private javax.swing.JMenuItem menuFileSaveScriptAs;
  private javax.swing.JSeparator jSeparator4;
  private javax.swing.JMenuItem menuFileExit;
  private javax.swing.JMenu menuScript;
  private javax.swing.JMenuItem menuScriptGraspBlock;
  private javax.swing.JMenuItem menuScriptHeater1On;
  private javax.swing.JMenuItem menuScriptHeater1Off;
  private javax.swing.JSeparator jSeparator3;
  private javax.swing.JMenu menuScriptSetLocation;
  private javax.swing.JMenuItem menuScriptSetLocation0;
  private javax.swing.JMenuItem menuScriptSetLocation1;
  private javax.swing.JMenuItem menuScriptSetLocation2;
  private javax.swing.JMenu menuScriptGoToLocation;
  private javax.swing.JMenuItem menuScriptGotoLocation0;
  private javax.swing.JMenuItem menuScriptGotoLocation1;
  private javax.swing.JMenuItem menuScriptGotoLocation2;
  private javax.swing.JSeparator jSeparator5;
  private javax.swing.JMenuItem menuScriptUsePythonConsole;
  private javax.swing.JMenuItem menuScriptAddFrame;
  private javax.swing.JMenuItem menuScriptRunScript;
  private javax.swing.JMenu menuStage;
  private javax.swing.JMenuItem menuStageGotoXYZ;
  private javax.swing.JMenuItem menuStageResetXY;
  private javax.swing.JMenuItem menuStageResetZ;
  private javax.swing.JSeparator jSeparator2;
  private javax.swing.JMenuItem menuStageSeekZSurface;
  private javax.swing.JMenu menuTweezers;
  private javax.swing.JMenuItem menuTweezersCalibrate;
  private javax.swing.JMenuItem menuTweezersResetArmOffsets;
  private javax.swing.JMenuItem menuTweezersResetTipOffsets;
  private javax.swing.JMenuItem menuTweezersGoto;
  private javax.swing.JSeparator jSeparator1;
  private javax.swing.JMenuItem menuTweezersIncreaseProbeXToContact;
  private javax.swing.JMenuItem menuTweezersDecreaseProbeXToContact;
  private javax.swing.JMenuItem menuTweezersIncreaseProbeYToContact;
  private javax.swing.JMenuItem menuTweezersDecreaseProbeYToContact;
  private javax.swing.JButton xDecrease;
  private javax.swing.JButton yIncrease;
  private javax.swing.JButton zDecrease;
  private javax.swing.JButton zIncrease;
  private javax.swing.JButton xIncrease;
  private javax.swing.JButton yDecrease;
  private javax.swing.JLabel _status;
  private javax.swing.JButton xIncYInc;
  private javax.swing.JButton xDecYInc;
  private javax.swing.JButton xIncYDec;
  private javax.swing.JButton xDecYDec;
  private javax.swing.JButton probeYInc;
  private javax.swing.JButton probeYDec;
  private javax.swing.JLabel jLabel2;
  private javax.swing.JButton probeXDec;
  private javax.swing.JButton probeXInc;
  private javax.swing.JLabel jLabel3;
  private javax.swing.JButton gotoXYZButton;
  private javax.swing.JButton probeXDecYDec;
  private javax.swing.JButton probeXIncYInc;
  private javax.swing.JButton probeXIncYDec;
  private javax.swing.JButton probeXDecYInc;
  private javax.swing.JButton gotoTweezerButton;
  private javax.swing.JButton runScriptButton;
  private javax.swing.JButton addFrameButton;
  private javax.swing.JButton consoleButton;
  // End of variables declaration//GEN-END:variables
  
  javax.swing.Timer _timer;
  private OrthoTweezersControl _control;
  private OrthoTweezersTools _tools;
  
  private TextAreaFrame _scriptEditor;
  private String _scriptDirectory, _scriptFilename;
  // This is used for the preferences
  static final String SCRIPT_DIRECTORY = "scriptdirectory";
  // This is used by checkScriptChangeOnDisk.
  long _scriptLastModified;
  
  private OrthoTweezersMonitor _monitor;
  // These are 0 for no motion, -1 for decreasing, 1 for increasing
  private int _moveX, _moveY, _moveZ;
  private int _moveProbeX, _moveProbeY;
  
  private File _preferencesFile;
  private Preferences _preferences = new Preferences ();
  
  public static final int MAX_LOCATIONS = 10;
  private Point3d[] _locations = new Point3d[MAX_LOCATIONS];
  private String[] _locationNames = new String[MAX_LOCATIONS];
  
  // The _console is created by establishConsole().
  private CancelInteractiveConsole _console;
}