    /*
    Copyright (c) 2000,2001. The Regents of the University of California
    (Regents). All Rights Reserved.
     
    Permission to use, copy, modify, and distribute this software and its
    documentation for educational, research,c and not-for-profit purposes,
    without fee and without a signed licensing agreement, is hereby
    granted, provided that the above copyright notice, this paragraph and
    the following two paragraphs appear in all copies, modifications, and
    distributions. Contact The Office of Technology Licensing, UC
    Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
    (510) 643-7201, for commercial licensing opportunities.
     
    IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
    INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
    LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
    DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
     
    REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
    A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
    ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
     */

package orthotweezers;

import java.io.*;
import java.text.NumberFormat;
import java.awt.*;

    /** Modeless dialog window for monitoring the orthotweezers system and
     * writing to the capture file.
     *
     * @author  unknown
     */
public class OrthoTweezersMonitor extends javax.swing.JDialog {
    
        /** Creates new form OrthoTweezersMonitor as a modeless dialog
         *
         * @param parent    the parent Frame
         * @param tools   the OrthoTweezersTools created by the main window.
         *               This calls tools.control() to get the OrthoTweezersControl 
         *                  which is queried for the values to display.
         */
    public OrthoTweezersMonitor(java.awt.Frame parent, OrthoTweezersTools tools) {
        super (parent, false);
        _tools = tools;
        _control = tools.control();
        initComponents ();
        // This overrides the value set in initComponents();
        monitorTimer.setDelay((long)TIMER_MILLIS);
    }
    
        /** This method is called from within the constructor to
         * initialize the form.
         * WARNING: Do NOT modify this code. The content of this method is
         * always regenerated by the FormEditor.
         */
    private void initComponents() {//GEN-BEGIN:initComponents
        monitorTimer = new org.netbeans.examples.lib.timerbean.Timer();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        _xArmDeflectionLabel = new javax.swing.JLabel();
        _yArmDeflectionLabel = new javax.swing.JLabel();
        _xTipForceLabel = new javax.swing.JLabel();
        _yTipForceLabel = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        _stageX = new javax.swing.JLabel();
        _stageY = new javax.swing.JLabel();
        _stageZ = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        _xTipDeflectionLabel = new javax.swing.JLabel();
        _yTipDeflectionLabel = new javax.swing.JLabel();
        _probesCanvas = new ProbesCanvas(this);
        _heater1State = new javax.swing.JLabel();
        monitorTimer.setDelay(10L);
        monitorTimer.addTimerListener(new org.netbeans.examples.lib.timerbean.TimerListener() {
            public void onTime(java.awt.event.ActionEvent evt) {
                monitorTimerOnTime(evt);
            }
        }
        );
        getContentPane().setLayout(null);
        setTitle("Ortho Tweezers Monitor");
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        
        jLabel1.setText("Arm Deflection (mm)");
        
        getContentPane().add(jLabel1);
        jLabel1.setBounds(0, 20, 120, 20);
        
        
        jLabel2.setText("Tip Force (mN)");
        
        getContentPane().add(jLabel2);
        jLabel2.setBounds(0, 60, 120, 20);
        
        
        jLabel3.setText("Probe X");
        
        getContentPane().add(jLabel3);
        jLabel3.setBounds(130, 0, 60, 20);
        
        
        jLabel4.setText("Probe Y");
        
        getContentPane().add(jLabel4);
        jLabel4.setBounds(220, 0, 50, 20);
        
        
        _xArmDeflectionLabel.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _xArmDeflectionLabel.setText("...");
        
        getContentPane().add(_xArmDeflectionLabel);
        _xArmDeflectionLabel.setBounds(120, 20, 80, 20);
        
        
        _yArmDeflectionLabel.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _yArmDeflectionLabel.setText("...");
        
        getContentPane().add(_yArmDeflectionLabel);
        _yArmDeflectionLabel.setBounds(200, 20, 80, 20);
        
        
        _xTipForceLabel.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _xTipForceLabel.setText("...");
        
        getContentPane().add(_xTipForceLabel);
        _xTipForceLabel.setBounds(120, 60, 80, 20);
        
        
        _yTipForceLabel.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _yTipForceLabel.setText("...");
        
        getContentPane().add(_yTipForceLabel);
        _yTipForceLabel.setBounds(200, 60, 80, 20);
        
        
        jLabel5.setText("stage X (mm)");
        
        getContentPane().add(jLabel5);
        jLabel5.setBounds(10, 90, 80, 20);
        
        
        jLabel6.setText("stage Y (mm)");
        
        getContentPane().add(jLabel6);
        jLabel6.setBounds(10, 110, 80, 20);
        
        
        jLabel7.setText("stage Z (mm)");
        
        getContentPane().add(jLabel7);
        jLabel7.setBounds(10, 130, 80, 20);
        
        
        _stageX.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _stageX.setText("...");
        
        getContentPane().add(_stageX);
        _stageX.setBounds(90, 90, 80, 20);
        
        
        _stageY.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _stageY.setText("...");
        
        getContentPane().add(_stageY);
        _stageY.setBounds(90, 110, 80, 20);
        
        
        _stageZ.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _stageZ.setText("...");
        
        getContentPane().add(_stageZ);
        _stageZ.setBounds(90, 130, 80, 20);
        
        
        jLabel8.setText("Tip Deflection (mm)");
        
        getContentPane().add(jLabel8);
        jLabel8.setBounds(0, 40, 120, 20);
        
        
        _xTipDeflectionLabel.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _xTipDeflectionLabel.setText("...");
        
        getContentPane().add(_xTipDeflectionLabel);
        _xTipDeflectionLabel.setBounds(120, 40, 80, 20);
        
        
        _yTipDeflectionLabel.setBorder(new javax.swing.border.LineBorder(java.awt.Color.black));
        _yTipDeflectionLabel.setText("...");
        
        getContentPane().add(_yTipDeflectionLabel);
        _yTipDeflectionLabel.setBounds(200, 40, 80, 20);
        
        
        _probesCanvas.setBackground(java.awt.Color.white);
        
        getContentPane().add(_probesCanvas);
        _probesCanvas.setBounds(110, 160, 120, 120);
        
        
        _heater1State.setText("Heater 1 OFF");
        
        getContentPane().add(_heater1State);
        _heater1State.setBounds(180, 90, 100, 20);
        
        setBounds(360, 0, 300, 320);
    }//GEN-END:initComponents
    
  private void monitorTimerOnTime (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_monitorTimerOnTime
      boolean doDisplay = false;
      boolean doCapture = false;
      
      if (++_displayCycles >= DISPLAY_MILLIS/TIMER_MILLIS) {
          // Should display this time
          doDisplay = true;
          _displayCycles = 0;
      }
      if (_captureWriter != null && _capture) {
          if (++_captureCycles >= _captureMillis/TIMER_MILLIS) {
              // Should capture this time
              doCapture = true;
              _captureCycles = 0;
          }
      }
      ++_cyclesSinceStartCapture;
      
      if (!(doDisplay || doCapture))
          // Nothing to do this time
          return;
      
      NumberFormat nf = NumberFormat.getInstance();
      nf.setMinimumFractionDigits (4);
      nf.setMaximumFractionDigits (4);
      
      // This reads the probe X and Y tip strain gauges twice, but do it anyway.
      // Get local copies of the forces and deflections which will be used by the
      //   numeric label displays and the canvas
      // Read the tip strains only once and convert to deflections and forces.
      double xTipStrain = _control.xTipStrain();
      double yTipStrain = _control.yTipStrain();
      _xTipDeflection = xTipStrain * _control.strainToDeflectionXTip();
      _xTipForce = xTipStrain * _control.strainToForceXTip();
      _yTipDeflection = yTipStrain * _control.strainToDeflectionYTip();
      _yTipForce = yTipStrain * _control.strainToForceYTip();
      _xArmDeflection = _control.xArmDeflection();
      _yArmDeflection = _control.yArmDeflection();
      
      if (doDisplay) {
          // Now set the numeric label display
          _xArmDeflectionLabel.setText (nf.format (_xArmDeflection));
          _yArmDeflectionLabel.setText (nf.format (_yArmDeflection));
          _xTipDeflectionLabel.setText (nf.format (_xTipDeflection));
          _yTipDeflectionLabel.setText (nf.format (_yTipDeflection));
          _xTipForceLabel.setText (nf.format (_xTipForce));
          _yTipForceLabel.setText (nf.format (_yTipForce));
          _stageX.setText (nf.format (_control.stageX()));
          _stageY.setText (nf.format (_control.stageY()));
          _stageZ.setText (nf.format (_control.stageZ()));
          _heater1State.setText (_tools.heaterState(1) ? "Heater 1 ON" : "Heater 1 OFF");
          
          // Redraw the probes position canvas
          _probesCanvas.repaint();
      }
      
      if (doCapture) {
          try {
              _captureWriter.write
              ((_cyclesSinceStartCapture * TIMER_MILLIS) + "\t" +
              nf.format(_xArmDeflection) + "\t" +
              nf.format(_xTipDeflection) + "\t" +
              nf.format(_xTipForce) + "\t" +
              nf.format(_yArmDeflection) + "\t" +
              nf.format(_yTipDeflection) + "\t" +
              nf.format(_yTipForce) + "\t" +
              nf.format(_control.stageX()) + "\t" +
              nf.format(_control.stageY()) + "\t" +
              nf.format(_control.stageZ()));
              _captureWriter.newLine();
          } catch (IOException e) {
          }
      }
  }//GEN-LAST:event_monitorTimerOnTime
  
  /** Open the file to which capture data is written.  After calling this,
   * you must call startCapture to begin writing capture data to the file.
   * If a capture file is already open, this stops any data being captured,
   * closes the file, and opens the given filename.
   *
   * @param filename    The full pathname of the file to capture data to.
   * @param append  if true, then data will be written to the end of the file.
   *        If false, previous data in the file is overwritten.
   * @return   true for success, false if the specified file is not found or
   *            if some other I/O error occurs
   * @see #startCapture
   */
  public boolean openCapture (String filename, boolean append) {
      // This will stop capturing and close the capture file if already open.
      closeCapture();
      
      try {
          _captureWriter = new BufferedWriter (new FileWriter (filename, append));
          
          if (!append) {
              // This is the first time the file is open, so write the header
              _captureWriter.write
              ("% time(ms)\txArmDef(mm)\txTipDef(mm)\txTipForce(mN)\tyArmDef(mm)\tyTipDef(mm)\tyTipForce(mN)\tstageX(mm)\tstageY(mm)\tstageZ(mm)");
              _captureWriter.newLine();
          }
      } catch (IOException e) {
          System.out.println(e);
          return false;
      }
      
      return true;
  }
  
  /** Write the given comment to the capture file.  If the capture file
   *   is not open, this does nothing (see openCapture).
   *
   * @param comment the comment to add to the capture file.  First, "% " is
   *     written before the comment, and a newline is written afterwards.
   * @see #openCapture
   */
  public void captureComment (String comment) {
      if (_captureWriter == null)
          // There is no capture file open.
          return;
      
      try {
          _captureWriter.write ("% " + comment);
          _captureWriter.newLine();
      } catch (IOException e) {
      }
  }
  
  /** Start capturing data with the given period and adding to the log file.
   * If already capturing, this changes the capture period to the given value.
   * All fields, such as stageX, xArmDeflection, etc. will be captured.
   * To stop capturing, call stopCapture() or closeCapture().
   *
   * @param period  the period in milliseconds between captures. If period is
   *            less than 10, this sets it to 10.
   *
   * @see #stopCapture
   */
  public void startCapture(int period) {
      if (_captureWriter == null)
          // There is no capture file open.
          return;
      
      if (period < TIMER_MILLIS)
          period = TIMER_MILLIS;
      
      _captureMillis = period;
      _captureCycles = 0;
      if (_capture == false)
          // Only reset the counter if we are not already capturing
          _cyclesSinceStartCapture = 0;
      _capture = true;
  }
  
  /** Stop capturing data to the capture file.  You may call startCapture() again
   * to resume capturing.
   *
   * @see #closeCapture
   */
  public void stopCapture() {
      _capture = false;
      
      if (_captureWriter != null) {
          // Even though we don't close the file now, flush the output buffer.
          try {
              _captureWriter.flush();
          } catch (IOException e) {
          }
      }
  }
  
  /** Stop capturing data to the capture file and close the capture file.
   * You only need to call closeCapture() if you wish to read the capture file
   * before the application exits (because the capture file is automatically
   * closed on exit).  Also, you do not need to call closeCapture() before opening
   * a new capture file (or re-opening the same file) since openCapture()
   * automatically does this.  If a capture file is not already open, this does
   * nothing.
   */
  public void closeCapture() {
      if (_captureWriter == null)
          // The capture file is not open.
          return;
      
      stopCapture();
      
      try {
          _captureWriter.close();
      } catch (IOException e) {
          // Debug: maybe we should report this error somehow.
      }
      
      _captureWriter = null;
  }
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private org.netbeans.examples.lib.timerbean.Timer monitorTimer;
  private javax.swing.JLabel jLabel1;
  private javax.swing.JLabel jLabel2;
  private javax.swing.JLabel jLabel3;
  private javax.swing.JLabel jLabel4;
  private javax.swing.JLabel _xArmDeflectionLabel;
  private javax.swing.JLabel _yArmDeflectionLabel;
  private javax.swing.JLabel _xTipForceLabel;
  private javax.swing.JLabel _yTipForceLabel;
  private javax.swing.JLabel jLabel5;
  private javax.swing.JLabel jLabel6;
  private javax.swing.JLabel jLabel7;
  private javax.swing.JLabel _stageX;
  private javax.swing.JLabel _stageY;
  private javax.swing.JLabel _stageZ;
  private javax.swing.JLabel jLabel8;
  private javax.swing.JLabel _xTipDeflectionLabel;
  private javax.swing.JLabel _yTipDeflectionLabel;
  private java.awt.Canvas _probesCanvas;
  private javax.swing.JLabel _heater1State;
  // End of variables declaration//GEN-END:variables
  
  private OrthoTweezersTools _tools;
  private OrthoTweezersControl _control;
  double _xTipDeflection, _xTipForce, _xArmDeflection;
  double _yTipDeflection, _yTipForce, _yArmDeflection;
  
  // The timer period in milliseconds
  private static int TIMER_MILLIS = 10;
  // The period in milliseconds for when the monitor should display.
  private static int DISPLAY_MILLIS = 100;
  // This counts timer cycles until the monitor should display.
  private int _displayCycles;
  
  // The period in milliseconds for capturing data.
  private int _captureMillis;
  // This counts timer cycles until capturing data.
  private int _captureCycles;
  // This increments since startCapture() to provide the time
  private int _cyclesSinceStartCapture;
  
  private boolean _capture;
  private BufferedWriter _captureWriter;
}

/** A ProbesCanvas is used to draw the probe positions on the _probesCanvas.
 * It is created by the automatic code generator for the Canvas because the
 * Code Generation property in the Forte component inspector
 * was changed to say new ProbesCanvas(this);
 * Otherwise, the ProbesCanvas is treated as a normal Canvas by calling
 * its repaint(), etc.
 */
class ProbesCanvas extends Canvas {
    /** Creates a new ProbesCanvas and calls super() for the base Canvas.
     *
     * @param monitor   the parent OrthoTweezersMonitor object from which
     *   the _xArmDeflection, etc. are read when paint() is drawing the probes.
     */
    public ProbesCanvas (OrthoTweezersMonitor monitor) {
        super();
        
        _monitor = monitor;
    }
    
    /** Paint the probes on the canvas.  This is usually invoked when the
     * timer calls repaint().
     */
    public void paint (Graphics graphics) {
        // Calculate the center of the canvas.  Use the _size object which
        //   is already allocated to avoid allocating new memory every time.
        getSize(_size);
        int centerX = _size.width / 2;
        int centerY = _size.height / 2;
        
        // We want 0.5 millimeters of deflection to move 50 pixels on the canvas, so
        //  multiply by 100.
        Point probeXTip = new Point
        (centerX + (int)((_monitor._xArmDeflection + _monitor._xTipDeflection) * 100.),
        centerY);
        Point probeYTip = new Point
        (centerX,
        centerY + (int)((_monitor._yArmDeflection + _monitor._yTipDeflection) * 100.));
        
        // Create temporary points.
        Point point1 = new Point();
        
        // Find the midpoint on the grip axis.
        Point axisMidpoint = new Point
        ((probeXTip.x + probeYTip.x) / 2, (probeXTip.y + probeYTip.y) / 2);
        
        // Draw the grip axis
        graphics.setColor (Color.green);
        drawLineProper (graphics, probeXTip, probeYTip);
        // Draw a line orthogonal to the grip axis
        int deltaX = -(probeXTip.y - probeYTip.y) / 2;
        int deltaY = (probeXTip.x - probeYTip.x) / 2;
        point1.setLocation (axisMidpoint);
        point1.translate (deltaX, deltaY);
        drawLineProper (graphics, point1, axisMidpoint);
        point1.setLocation (axisMidpoint);
        point1.translate (-deltaX, -deltaY);
        drawLineProper (graphics, point1, axisMidpoint);
        
        // The total grip force is reflected across the line x=y, so the x component shows
        //   the force from probe Y and vice versa.  We want 10 millinewtons to be
        //   50 pixels, so multiply by 5.
        point1.setLocation (axisMidpoint);
        point1.translate ((int)(_monitor._yTipForce * 5.), (int)(_monitor._xTipForce * 5.));
        graphics.setColor (Color.red);
        drawLineProper (graphics, point1, axisMidpoint);
        // Draw individual lines of force for each probe. Negate the force so it is
        //   force on the object, not vice versa.
        point1.setLocation (probeXTip);
        point1.translate ((int)(- _monitor._xTipForce * 5.), 0);
        drawLineProper (graphics, point1, probeXTip);
        point1.setLocation (probeYTip);
        point1.translate (0, (int)(- _monitor._yTipForce * 5.));
        drawLineProper (graphics, point1, probeYTip);
        
        // Draw each probe
        graphics.setColor (Color.black);
        point1.setLocation (probeXTip.x, 0);
        drawLineProper (graphics, point1, probeXTip);
        point1.setLocation (0, probeYTip.y);
        drawLineProper (graphics, point1, probeYTip);
    }
    
    /** Since larger screen coordinates are lower on the screen, but we want proper
     *  axes where they are higher on the screen so draw the line after inverting each point's Y
     *  as (_size.height - y).  Assume that _size has already been set to getSize (_size).
     *
     * @graphics    the Graphics context
     * @param point1    the beginning point of the line
     * @param point2    the end point of the line
     */
    private void drawLineProper (Graphics graphics, Point point1, Point point2) {
        graphics.drawLine (point1.x, _size.height - point1.y,
        point2.x, _size.height - point2.y);
    }
    
    private OrthoTweezersMonitor _monitor;
    private Dimension _size = new Dimension();
}