/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved.
 
Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
 
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
 
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
 */

package orthotweezers;

import java.io.*;
import org.python.util.*; 
import org.python.core.*; 

/** RunScript is a modal dialog box which runs the given Python script
 * and provides a Cancel button to interrupt it.  Before running the
 * script, this always sets up the Python variables control and tools
 * as the OrthoTweezersControl and OrthoTweezersTools, repectively.
 *
 * @author  unknown
 */
public class RunScript extends ExecuteDialog {

    /** Creates new modal RunScript dialog box.  When the user presses
     *    the Run button, this runs the given Python script.
     *    The algorithm is run in a separate
     *    thread and the user can cancel.
     *
     * @param parent    The parent Frame
     * @param console   The InteractiveConsole which has already been set up
     *          with the global variables control, tools and the easy access
     *          functions as in OrthoTweezers.establishConsole().
     * @param script    The script the run, such as returned by ScriptEditor.getText().
     */
    public RunScript(java.awt.Frame parent, InteractiveConsole console, String script) {
        super (parent, "Run Script", "Run", "Click Run to begin");

        _console = console;
        _script = script;
    }
    
    /** This is called when the new thread is created to execute the procedure.
     */
    public void execute() throws CancelException {
        try {            
            setStatus ("Running python script - See console window");
            System.out.println("Running Python script...");
            
            // We have a String and want an InputStream.  The class
            //  StringBufferInputStream has been deprecated, so we have
            //  to use a ByteArrayInputStream.  This means we have to
            //  convert all of the String to a byte array up front which
            //  is a waste of memory. 
            _console.execfile (new ByteArrayInputStream (_script.getBytes()));

            setStatus ("done");
            System.out.println("Done.");
        } catch (org.python.core.PyException pythonException) {
            // First test whether this is a CancelException
            Object object = pythonException.value.__tojava__(Object.class);
            if (object instanceof CancelException) {
                CancelException cancelException = (CancelException)object;
                System.out.println(cancelException.getMessage());
                // We don't need to setStatus because the dialog box will close immediately
                throw cancelException;
            }
            
            //Otherwise, send the error to the console window
            setStatus ("Python ERROR - See console window");
            System.out.println("Python ERROR:\n" + pythonException);
        }
    }

    private InteractiveConsole _console;
    private String _script;
}
