/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved.
 
Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
 
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
 
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
 */

package orthotweezers;

import java.awt.*;

/** A TextAreaFrame provides a text area in a JFrame window for viewing and editing text.
 * The JTextArea is available from textArea() and you can call textArea().setText()
 * or textArea().getText() to set and get the text.
 *
 * @author  unknown
 */
public class TextAreaFrame extends javax.swing.JFrame {
    
    /** Creates new form ScriptEditor */
    public TextAreaFrame() {
        initComponents ();
        pack ();
        
        _documentListener = new MyDocumentListener();
        _textArea.getDocument().addDocumentListener(_documentListener);
    }
    
    // This is an inner class
    class MyDocumentListener implements javax.swing.event.DocumentListener {
            public void removeUpdate(javax.swing.event.DocumentEvent event) {
                _isModified = true;
            }
            public void insertUpdate(javax.swing.event.DocumentEvent event) {
                _isModified = true;
            }
            public void changedUpdate(javax.swing.event.DocumentEvent event) {
                // changedUpdate means a property, not the text, has changed, so ignore.
            }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        _scrollPane = new javax.swing.JScrollPane();
        _textArea = new javax.swing.JTextArea();
        getContentPane().setLayout(new java.awt.FlowLayout());
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        }
        );
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        }
        );
        
        _scrollPane.setPreferredSize(new java.awt.Dimension(500, 300));
        _scrollPane.setMinimumSize(new java.awt.Dimension(500, 300));
        
        _textArea.setTabSize(2);
          _textArea.setFont(new java.awt.Font ("Courier New", 0, 11));
          _textArea.addFocusListener(new java.awt.event.FocusAdapter() {
              public void focusGained(java.awt.event.FocusEvent evt) {
                  _textAreaFocusGained(evt);
              }
          }
          );
          _textArea.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
              public void propertyChange(java.beans.PropertyChangeEvent evt) {
                  _textAreaPropertyChange(evt);
              }
          }
          );
          _scrollPane.setViewportView(_textArea);
          
          
        getContentPane().add(_scrollPane);
        
        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        java.awt.Dimension dialogSize = getSize();
        setSize(new java.awt.Dimension(400, 200));
        setLocation((screenSize.width-400)/2,(screenSize.height-200)/2);
    }//GEN-END:initComponents

  private void _textAreaPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event__textAreaPropertyChange
      // A property change happens if the document associated with the text area
      // changes, so make sure the currently attached document has our listner.
      // It is okay to call remove even if it isn't in there yet.
      _textArea.getDocument().removeDocumentListener(_documentListener);
      _textArea.getDocument().addDocumentListener(_documentListener);
  }//GEN-LAST:event__textAreaPropertyChange
    
  private void _textAreaFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event__textAreaFocusGained
      // Just call the public version of this method which can be overridden
      // by a subclass.
      focusGained(evt);
  }//GEN-LAST:event__textAreaFocusGained
  
  private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
      Dimension size =
      new Dimension (getContentPane().getWidth() - 6, getContentPane().getHeight() - 6);
      _scrollPane.setPreferredSize (size);
      // Not sure why, but if we don't also call setSize here, the change doesn't
      //   happen during the validate().
      _scrollPane.setSize (size);
      getContentPane().validate();
  }//GEN-LAST:event_formComponentResized
  
  private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
      // Just call the public version of this method which can be overridden
      // by a subclass.
      windowClosing(evt);
  }//GEN-LAST:event_formWindowClosing
  
  /** Return the JTextArea containing the text.  For example, to get the
   * text, call textArea().getText().
   */
  public javax.swing.JTextArea textArea() {
      return _textArea;
  }
  
  /** Return true if the text has been modified since the last save or
   * since being set to new text.
   */
  public boolean isModified() {
      return _isModified;
  }
  
  /** Sets the modified state to false.  You should call this after opening
   * a new file and setting the text because the text does not need to
   * be changed at this point.
   */
  public void clearModified() {
      _isModified = false;
  }
  
   /** This is called when the TextAreaFrame receives a windowClosing message.
    * The default method just calls setVisible(false), but you can override it,
    * for example, to check whether it is okay to close.
    * If this method does nothing, the window is not closed.
    */
  public void windowClosing(java.awt.event.WindowEvent evt) {
      setVisible (false);
  }
  
   /** This is called upon the text area focus gained.  The default method
    * does nothing, but you can override it to check something once the 
    * focus is gained and before the user makes more changes.
    */
  public void focusGained(java.awt.event.FocusEvent evt) {
  }
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JScrollPane _scrollPane;
  private javax.swing.JTextArea _textArea;
  // End of variables declaration//GEN-END:variables
  
  private boolean _isModified;
  private javax.swing.event.DocumentListener _documentListener;
}
