/*
Copyright (c) 2000,2001. The Regents of the University of California
(Regents). All Rights Reserved.
 
Permission to use, copy, modify, and distribute this software and its
documentation for educational, research, and not-for-profit purposes,
without fee and without a signed licensing agreement, is hereby
granted, provided that the above copyright notice, this paragraph and
the following two paragraphs appear in all copies, modifications, and
distributions. Contact The Office of Technology Licensing, UC
Berkeley, 2150 Shattuck Avenue, Suite 510, Berkeley, CA 94720-1620,
(510) 643-7201, for commercial licensing opportunities.
 
IN NO EVENT SHALL REGENTS BE LIABLE TO ANY PARTY FOR DIRECT,
INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION, EVEN IF REGENTS HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
 
REGENTS SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF
ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS". REGENTS HAS NO
OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
MODIFICATIONS.
 */

package orthotweezers;

import java.io.*;
import org.python.util.*; 
import org.python.core.*; 

/** UsePythonConsole is a modal dialog box which allows the user to type Python
 * commands and view output in the command console which invoked Java. 
 * The dialog box provides a Cancel button to interrupt it.  
 * The Python variables control and tools are set to
 * the OrthoTweezersControl and OrthoTweezersTools objects, repectively.
 *
 * @author  unknown
 */
public class UsePythonConsole extends ExecuteDialog {

    /** Creates new modal UsePythonConsole dialog box.  
     *    The console is run in a separate
     *    thread and the user can cancel.
     *
     * @param parent    The parent Frame
     * @param console   The CancelInteractiveConsole which has already been set up
     *          with the global variables control, tools and the easy access
     *          functions.
     */
    public UsePythonConsole(java.awt.Frame parent, CancelInteractiveConsole console) {
        // Indicate that we want to execute immediately
        super (parent, "Use Python Console", "Use Console", "Click Use Console to begin", true);
        _console = console;
        setCancelMessage ("Cancelling... Normally, you should type 'exit' to exit the console.");
    }

    /** This is called when the new thread is created to execute the procedure.
     */
    public void execute() throws CancelException {
        setStatus ("Enter commands in console. Enter 'exit' when done.");

        try {
            // Make sure the input buffer is clear
            _console.resetbuffer();
            
            // Flush any typing the user has made up to now.
            _console.flushReader();
            _console.interact();        
            setStatus ("done");
        } catch (org.python.core.PyException pythonException) {
            // First test whether this is a CancelException
            Object object = pythonException.value.__tojava__(Object.class);
            if (object instanceof CancelException) {
                CancelException cancelException = (CancelException)object;
                System.out.println(cancelException.getMessage());
                // We don't need to setStatus because the dialog box will close immediately
                throw cancelException;
            }
            
            // Otherwise, send the error to the console window.
            // This shouldn't really happen since interact() catchs most and
            //   just prints them, allowing the user to resume interacting.
            setStatus ("Python ERROR - See console window");
            System.out.println("Python ERROR:\n" + pythonException);
        }
    }
    
    private CancelInteractiveConsole _console;
}
